/**
  * @license SVOGV v0.3.7
  * Copyright (c) 2011-2018 Joerg <IsAGeek> Krause, Berlin https://www.joergkrause.de/
  * License: ICS
  */
(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/forms'), require('@angular/platform-browser'), require('@angular/router'), require('@svogv/core')) :
	typeof define === 'function' && define.amd ? define(['exports', '@angular/core', '@angular/forms', '@angular/platform-browser', '@angular/router', '@svogv/core'], factory) :
	(factory((global.ac = global.ac || {}, global.ac.svogv = global.ac.svogv || {}),global.ng.core,global.ng.forms,global.ng.platformBrowser,global.ng.router,global.svogv.core));
}(this, (function (exports,_angular_core,_angular_forms,_angular_platformBrowser,_angular_router,d) { 'use strict';

var EnumConverter = function (value, enumerationType) {
    var sanitizedValue = value.toString();
    var color = (enumerationType[sanitizedValue]);
    return color.toString();
};
var StringConverter = function (value) {
    if (value === null || value === undefined || typeof value === 'string') {
        return value;
    }
    return value.toString();
};
var BooleanConverter = function (value) {
    if (value === null || value === undefined || typeof value === 'boolean') {
        return value;
    }
    return value.toString() === 'true';
};
var NumberConverter = function (value) {
    if (value === null || value === undefined || typeof value === 'number') {
        return value;
    }
    return parseFloat(value.toString());
};
function InputConverter(converter, enumerationType) {
    return function (target, key) {
        if (converter === undefined) {
            var metadata = Reflect.getMetadata('design:type', target, key);
            if (metadata === undefined || metadata === null) {
                throw new Error('The reflection metadata could not be found.');
            }
            switch (metadata.name) {
                case 'String':
                    converter = StringConverter;
                    break;
                case 'Boolean':
                    converter = BooleanConverter;
                    break;
                case 'Number':
                    converter = NumberConverter;
                    break;
                default:
                    throw new Error('There is no converter for the given property type \'' + metadata.name + '\'.');
            }
        }
        var definition = Object.getOwnPropertyDescriptor(target, key);
        if (definition) {
            Object.defineProperty(target, key, {
                get: definition.get,
                set: function (newValue) {
                    if (enumerationType) {
                        definition.set(converter(newValue, enumerationType));
                    }
                    else {
                        definition.set(converter(newValue));
                    }
                },
                enumerable: true,
                configurable: true
            });
        }
        else {
            // create a helper to access the converter
            Object.defineProperty(target, key, {
                get: function () {
                    return this['__' + key];
                },
                set: function (newValue) {
                    this['__' + key] = enumerationType === undefined ? converter(newValue) : converter(newValue, enumerationType);
                },
                enumerable: false,
                configurable: true
            });
        }
    };
}

/**
 * Additional options for the control.
 * If not provided the control assumes the values from other settings.
 */
var AcInfoBoxOptions = (function () {
    function AcInfoBoxOptions() {
        this.hasProgress = false;
        this.hasFooter = false;
    }
    return AcInfoBoxOptions;
}());

/**
 * Colors specific to Boostrap.
 */
/**
 * Colors specific to Boostrap.
 */ var Meaning;
(function (Meaning) {
    Meaning[Meaning["Success"] = 0] = "Success";
    Meaning[Meaning["Info"] = 1] = "Info";
    Meaning[Meaning["Warning"] = 2] = "Warning";
    Meaning[Meaning["Danger"] = 3] = "Danger";
})(Meaning || (Meaning = {}));
/**
 * Semantic Meaning of buttons, specific to Bootstrap
 */
var Actions;
(function (Actions) {
    Actions[Actions["Default"] = 0] = "Default";
    Actions[Actions["Primary"] = 1] = "Primary";
    Actions[Actions["Secondary"] = 2] = "Secondary";
    Actions[Actions["Success"] = 3] = "Success";
    Actions[Actions["Info"] = 4] = "Info";
    Actions[Actions["Warning"] = 5] = "Warning";
    Actions[Actions["Danger"] = 6] = "Danger";
})(Actions || (Actions = {}));
/**
 * Sizes of menus and buttons
 */
var Sizes;
(function (Sizes) {
    Sizes[Sizes["XSmall"] = 0] = "XSmall";
    Sizes[Sizes["Small"] = 1] = "Small";
    Sizes[Sizes["Medium"] = 2] = "Medium";
    Sizes[Sizes["Large"] = 3] = "Large";
    Sizes[Sizes["XLarge"] = 4] = "XLarge";
})(Sizes || (Sizes = {}));

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d$$1;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d$$1 = decorators[i]) r = (c < 3 ? d$$1(r) : c > 3 ? d$$1(target, key, r) : d$$1(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
/**
 * The InfoBox is a rectangle area to show dynamic data. It has icon, text, and additional information.
 *
 * @param icon The name of an FontAwesome icon, such as "fa-circle"
 * @param text The text that appears prominently
 * @param number A number that covers the element as the value
 * @param footerText A smaller explanation text at the end
 * @param footerLink A link, suitable for [routerLink] that is being invoked from the footerText
 * @param progressValue A progress Value that creates a progress bar position
 * @param progressText A progress Text that replaces the footerText
 * @param color The background color
 */
exports.AcInfoBox = (function () {
    function AcInfoBox() {
        this.color = Meaning.Info;
        this.text = 'Demo';
        this.progressValue = 0;
        this.progressText = '';
        this.footerLink = '/';
        this.footerText = '';
        this.icon = 'fa-user';
    }
    AcInfoBox.prototype.ngOnInit = function () {
        if (!this.options) {
            this.options = new AcInfoBoxOptions();
            this.options.hasFooter = !!this.footerLink || !!this.footerText;
            this.options.hasProgress = !!this.progressValue || !!this.progressText;
        }
    };
    AcInfoBox.prototype.getColor = function (type) {
        if (this.color) {
            var color = EnumConverter(this.color, Meaning).toLowerCase();
            return type + "-" + color;
        }
    };
    return AcInfoBox;
}());
__decorate([
    _angular_core.Input(),
    __metadata("design:type", String)
], exports.AcInfoBox.prototype, "icon", void 0);
__decorate([
    _angular_core.Input(),
    __metadata("design:type", String)
], exports.AcInfoBox.prototype, "text", void 0);
__decorate([
    _angular_core.Input(),
    __metadata("design:type", String)
], exports.AcInfoBox.prototype, "number", void 0);
__decorate([
    _angular_core.Input(),
    __metadata("design:type", String)
], exports.AcInfoBox.prototype, "footerText", void 0);
__decorate([
    _angular_core.Input(),
    __metadata("design:type", String)
], exports.AcInfoBox.prototype, "footerLink", void 0);
__decorate([
    _angular_core.Input(),
    __metadata("design:type", Number)
], exports.AcInfoBox.prototype, "progressValue", void 0);
__decorate([
    _angular_core.Input(),
    __metadata("design:type", String)
], exports.AcInfoBox.prototype, "progressText", void 0);
__decorate([
    _angular_core.Input(),
    InputConverter(EnumConverter, Meaning) // need EnumConverter, as otherwise metadata returns Number
    ,
    __metadata("design:type", Number)
], exports.AcInfoBox.prototype, "color", void 0);
__decorate([
    _angular_core.Input(),
    __metadata("design:type", AcInfoBoxOptions)
], exports.AcInfoBox.prototype, "options", void 0);
exports.AcInfoBox = __decorate([
    _angular_core.Component({
        selector: 'ac-infobox',
        template: "<div class=\"card card-inverse\" [ngClass]=\"getColor('card')\">\n                  <div class=\"card-body\" [ngClass]=\"getColor('bg')\">\n                    <div class=\"rotate\">\n                        <i class=\"fa fa-5x\" [ngClass]=\"icon\"></i>\n                    </div>\n                    <h6 class=\"text-uppercase\">{{ text }}</h6>\n                    <h1 class=\"display-1\">{{ number }}</h1>\n                    <div class=\"progress\" *ngIf=\"options.hasProgress\">\n                        <div class=\"progress-bar\" [style.width]=\"progressValue\"></div>\n                    </div>\n                    <span class=\"progress-description\" *ngIf=\"options.hasProgress\">\n                        {{progressText}}\n                    </span>\n                  </div>\n                  <div class=\"card-footer text-muted\" *ngIf=\"options.hasFooter\">\n                      <a href=\"#\" class=\"card-link\" [routerLink]=\"footerLink\">{{footerText}}</a>\n                  </div>\n                </div>"
    }) //
    ,
    __metadata("design:paramtypes", [])
], exports.AcInfoBox);

var __decorate$1 = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d$$1;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d$$1 = decorators[i]) r = (c < 3 ? d$$1(r) : c > 3 ? d$$1(target, key, r) : d$$1(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata$1 = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
/**
 * The pagination component creates a few buttons to navigate a grid. The underlaying model
 * is going to handle the date on the client. The pagination does not support a server backend,
 * all relevant data must be loaded first.
 */
exports.AcDataGridPagination = (function () {
    function AcDataGridPagination() {
        /**
         * An event fired once the user has changed the page by clicking a button.
         */
        this.pageNumberChanged = new _angular_core.EventEmitter();
        this.currentPageNumber = 1;
    }
    AcDataGridPagination.prototype.ngOnInit = function () {
        this.setCurrentPage(1);
    };
    AcDataGridPagination.prototype.ngOnChanges = function (changes) {
        var _this = this;
        if (changes['maxPageIndex']) {
            var change = changes['maxPageIndex'];
            if (this.currentPageNumber > change.currentValue) {
                // throws ExpressionChangedAfterItHasBeenCheckedException
                // if there's no setTimeout.
                // no need to add setTimeout if ngOnChanges 
                // is fired after changes made on root component.
                setTimeout(function () { return _this.setCurrentPage(1); }, 1);
            }
        }
    };
    AcDataGridPagination.prototype.setCurrentPage = function (pageNumber, event) {
        if (event) {
            event.preventDefault();
        }
        if (pageNumber === 0 || pageNumber > this.maxPageIndex
            || pageNumber === this.currentPageNumber) {
            return;
        }
        this.pageNumberChanged.emit(pageNumber);
        this.currentPageNumber = pageNumber;
    };
    AcDataGridPagination.prototype.range = function (min, max) {
        var result = new Array();
        for (var i = min; i <= max; i++) {
            result.push(i);
        }
        return result;
    };
    Object.defineProperty(AcDataGridPagination.prototype, "pageStartNumber", {
        get: function () {
            var startNumber = this.currentPageNumber <= 4
                ? 1
                : this.currentPageNumber >= this.maxPageIndex - 3
                    ? this.maxPageIndex - 6
                    : this.currentPageNumber - 3;
            return startNumber < 1 ? 1 : startNumber;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AcDataGridPagination.prototype, "pageEndNumber", {
        get: function () {
            var pageEnd = this.pageStartNumber + 6;
            return pageEnd > this.maxPageIndex ? this.maxPageIndex : pageEnd;
        },
        enumerable: true,
        configurable: true
    });
    return AcDataGridPagination;
}());
__decorate$1([
    _angular_core.Input(),
    __metadata$1("design:type", Number)
], exports.AcDataGridPagination.prototype, "maxPageIndex", void 0);
__decorate$1([
    _angular_core.Output(),
    __metadata$1("design:type", Object)
], exports.AcDataGridPagination.prototype, "pageNumberChanged", void 0);
exports.AcDataGridPagination = __decorate$1([
    _angular_core.Component({
        selector: 'ac-pagination',
        styles: [
            ".pagination {\n      display: inline-block;\n      padding-left: 0;\n      margin: 20px 0;\n      border-radius: 4px;\n    }",
            ".pagination > li {\n      display: inline;\n    }",
            ".pagination > li:first-child > span {\n      margin-left: 0;\n      border-bottom-left -radius: 4px;\n      border-top-left-radius: 4px;\n    }",
            ".pagination >.disabled > a, .pagination >.disabled > a:focus, .pagination >.disabled > a:hover, .pagination >.disabled > span, .pagination >.disabled > span:focus, .pagination >.disabled > span:hover {\n      color: #777;\n      background-color: #fff;\n      border-color: #ddd;\n      cursor: not-allowed;\n    }",
            ".pagination > li > a, .pagination > li > span {\n      position: relative;\n      float: left;\n      padding: 6px 12px;\n      line-height: 1.42857;\n      text-decoration: none;\n      color: #337ab7;\n      background-color: #fff;\n      border: 1px solid #ddd;\n      margin-left: -1px;\n    }"
        ],
        template: "\n    <div>\n        <ul class=\"pagination float-right\">\n            <li [class.disabled]=\"currentPageNumber === 1 || !maxPageIndex\">\n                <a href (click)=\"setCurrentPage(1, $event)\" aria-label=\"Previous\">\n                    <span aria-hidden=\"true\">\u00AB</span>\n                </a>\n            </li>\n            <li [class.disabled]=\"currentPageNumber === 1 || !maxPageIndex\">\n                <a href aria-label=\"Previous\" \n                    (click)=\"setCurrentPage(currentPageNumber - 1, $event)\">\n                    <span aria-hidden=\"true\">\u2039</span>\n                </a>\n            </li>\n            <li *ngFor=\"let index of range(pageStartNumber, pageEndNumber)\" \n                        [class.active]=\"currentPageNumber === index\">\n                <a href (click)=\"setCurrentPage(index, $event)\">\n                    <span aria-hidden=\"true\">{{ index }}</span>\n                </a>\n            </li>\n            <li [class.disabled]=\"currentPageNumber === maxPageIndex \n                                  || !maxPageIndex\">\n                <a href (click)=\"setCurrentPage(currentPageNumber + 1, $event)\" aria-label=\"Last\">\n                    <span aria-hidden=\"true\">\u203A</span>\n                </a>\n            </li>\n            <li [class.disabled]=\"currentPageNumber === maxPageIndex \n                                  || !maxPageIndex\">\n                <a href (click)=\"setCurrentPage(maxPageIndex, $event)\" aria-label=\"Last\">\n                    <span aria-hidden=\"true\">\u00BB</span>\n                </a>\n            </li>\n        </ul>\n    </div>    \n    "
    })
], exports.AcDataGridPagination);

var __decorate$2 = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d$$1;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d$$1 = decorators[i]) r = (c < 3 ? d$$1(r) : c > 3 ? d$$1(target, key, r) : d$$1(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata$2 = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
/**
 * This pipe allows the dynamic formatting of data in forms or grid by applying
 * the @Format(PipeName) decorator.
 *
 * Usage within the form:
 *
 * @example {{ data | formatData:model }}
 *
 * If the model has a decorator for the specific property 'data' refers to, it will pull
 * the actual Pipe from there:
 *
 * @example @Format(PercentagePipe)
 *          percent: number = 0;
 *
 * The module must have access to <code>PercentPipe</code> and the dynamic pipe must have
 * access to a model that's a real instance (interfaces will not work). This is because
 * we need to reflect the meta data from hidden properties that exists for sure.
 *
 */
exports.AcFormatDataPipe = (function () {
    function AcFormatDataPipe(injector) {
        this.injector = injector;
    }
    AcFormatDataPipe.prototype.transform = function (value, pipeToken, pipeArgs) {
        if (!pipeToken) {
            return value;
        }
        else {
            if (pipeArgs && pipeArgs.length >= 2) {
                // First arg is the model
                var model = pipeArgs[0];
                var pipe = this.injector.get(pipeToken);
                return pipe.transform.apply(pipe, [value].concat(pipeArgs));
            }
            else {
                console.log('Illegal usage of "formatData" Pipe. Use two parameters like this: {{ data | formatData:model:propNames }}');
                return value;
            }
        }
    };
    return AcFormatDataPipe;
}());
exports.AcFormatDataPipe = __decorate$2([
    _angular_core.Pipe({
        name: 'formatData'
    }),
    __metadata$2("design:paramtypes", [_angular_core.Injector])
], exports.AcFormatDataPipe);

var __decorate$3 = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d$$1;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d$$1 = decorators[i]) r = (c < 3 ? d$$1(r) : c > 3 ? d$$1(target, key, r) : d$$1(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata$3 = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
/**
 * A circling symbol with various settings to control speed and color.
 */
exports.AcLoaderIcon = (function () {
    function AcLoaderIcon() {
        this.rotateSpeed = 1;
        this.color = '#FF0000';
    }
    AcLoaderIcon.prototype.ngAfterViewInit = function () {
        var native = this.group.nativeElement;
        var x = 0;
        var y = 12;
        setInterval(function () {
            for (var i = 0; i < native.children.length; i++, y--) {
                x = (y - i) * 30 - 360;
                native.children[i].setAttribute('transform', "rotate(" + x + " 18 18)");
                if (y === 12) {
                    y = 0;
                }
            }
        }, (100 / this.rotateSpeed * 13));
    };
    return AcLoaderIcon;
}());
__decorate$3([
    _angular_core.Input(),
    __metadata$3("design:type", Number)
], exports.AcLoaderIcon.prototype, "rotateSpeed", void 0);
__decorate$3([
    _angular_core.Input(),
    __metadata$3("design:type", String)
], exports.AcLoaderIcon.prototype, "color", void 0);
__decorate$3([
    _angular_core.ViewChild('group'),
    __metadata$3("design:type", _angular_core.ElementRef)
], exports.AcLoaderIcon.prototype, "group", void 0);
exports.AcLoaderIcon = __decorate$3([
    _angular_core.Component({
        selector: 'ac-loadericon',
        template: "<svg width=\"36px\" height=\"36px\" viewBox=\"0 0 36 36\" version=\"1.1\" xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\"><g id=\"Page-1\" stroke=\"none\" stroke-width=\"1\" fill=\"none\" fill-rule=\"evenodd\"><g id=\"Group-3\" [attr.fill]=\"color\" #group><rect id=\"Rectangle-1\" x=\"16.5873418\" y=\"0\" width=\"3\" height=\"9.13705584\"></rect><rect id=\"Rectangle-2\" fill-opacity=\"0.58\" x=\"16.678481\" y=\"26.8629442\" width=\"3\" height=\"9.13705584\"></rect><rect id=\"Rectangle-4\" fill-opacity=\"0.79\" transform=\"translate(31.530380, 17.954315) rotate(-270.000000) translate(-31.530380, -17.954315) \" x=\"30.0303797\" y=\"13.3857868\" width=\"3\" height=\"9.13705584\"></rect><rect id=\"Rectangle-5\" fill-opacity=\"0.37\" transform=\"translate(4.735443, 18.045685) rotate(-270.000000) translate(-4.735443, -18.045685) \" x=\"3.23544304\" y=\"13.4771574\" width=\"3\" height=\"9.13705584\"></rect><rect id=\"Rectangle-6\" fill-opacity=\"0.72\" transform=\"translate(29.758244, 24.676171) rotate(-240.000000) translate(-29.758244, -24.676171) \" x=\"28.2582441\" y=\"20.1076435\" width=\"3\" height=\"9.13705584\"></rect><rect id=\"Rectangle-7\" fill-opacity=\"0.3\" transform=\"translate(6.507579, 11.323829) rotate(-240.000000) translate(-6.507579, -11.323829) \" x=\"5.00757864\" y=\"6.75530065\" width=\"3\" height=\"9.13705584\"></rect><rect id=\"Rectangle-8\" fill-opacity=\"0.65\" transform=\"translate(24.871110, 29.609153) rotate(-210.000000) translate(-24.871110, -29.609153) \" x=\"23.37111\" y=\"25.0406255\" width=\"3\" height=\"9.13705584\"></rect><rect id=\"Rectangle-9\" fill-opacity=\"0.23\" transform=\"translate(11.394713, 6.390847) rotate(-210.000000) translate(-11.394713, -6.390847) \" x=\"9.89471277\" y=\"1.82231869\" width=\"3\" height=\"9.13705584\"></rect><rect id=\"Rectangle-10\" fill-opacity=\"0.51\" transform=\"translate(11.473642, 29.654839) rotate(-150.000000) translate(-11.473642, -29.654839) \" x=\"9.97364166\" y=\"25.0863108\" width=\"3\" height=\"9.13705584\"></rect><rect id=\"Rectangle-11\" fill-opacity=\"0.93\" transform=\"translate(24.792181, 6.345161) rotate(-150.000000) translate(-24.792181, -6.345161) \" x=\"23.2921811\" y=\"1.77663341\" width=\"3\" height=\"9.13705584\"></rect><rect id=\"Rectangle-12\" fill-opacity=\"0.44\" transform=\"translate(6.553148, 24.755301) rotate(-120.000000) translate(-6.553148, -24.755301) \" x=\"5.05314826\" y=\"20.1867727\" width=\"3\" height=\"9.13705584\"></rect><rect id=\"Rectangle-13\" fill-opacity=\"0.86\" transform=\"translate(29.712675, 11.244699) rotate(-120.000000) translate(-29.712675, -11.244699) \" x=\"28.2126745\" y=\"6.67617143\" width=\"3\" height=\"9.13705584\"></rect></g></g></svg>"
    }),
    __metadata$3("design:paramtypes", [])
], exports.AcLoaderIcon);

var __decorate$4 = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d$$1;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d$$1 = decorators[i]) r = (c < 3 ? d$$1(r) : c > 3 ? d$$1(target, key, r) : d$$1(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata$4 = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
/**
 * The Editor Widget. Creates a field with all required validators using decorators and forms service.
 */
exports.AcEditor = (function () {
    function AcEditor() {
        /**
         * Editor type. Default is 'text';
         */
        this.type = 'text';
        /**
         * A character after the fields label. Default is ': ' (colon plus space);
         */
        this.labelDivider = ': ';
        /**
         * If set to true the label and the field appears in one row.
         * Otherwise the label is above the field. Default is `true`.
         */
        this.inline = true;
        /**
         * The start value for a range field. Other field types ignore this value.
         */
        this.fromValue = 0;
        /**
         * The end value for a range field. Other field types ignore this value.
         */
        this.toValue = 100;
        /**
         * An optional placeholder for empty field. The default is empty (no watermark).
         */
        this.waterMark = '';
        /**
         * Renders the field as read only.
         */
        this.readonly = false;
        this.params = new Array();
    }
    AcEditor.prototype.getParams = function (key) {
        return this.first(this.params.filter(function (e) { return e.key === key; }), 0);
    };
    AcEditor.prototype.first = function (array, n) {
        if (array == null) {
            return void 0;
        }
        if (n == null) {
            return array[0];
        }
        if (n < 0) {
            return [];
        }
        return array.slice(0, n);
    };
    AcEditor.prototype.ngOnInit = function () {
        var _this = this;
        this.userForm.valueChanges.subscribe(function (data) { return _this.onValueChanged(data); });
        // this is set by FormValidatorService
        var editorModel = this.userForm['__editorModel__'];
        // get type from form
        if (editorModel) {
            // get elementary types, this might get overwritten later according to decorators found
            if (typeof editorModel[this.name] === 'string') {
                this.type = 'text';
            }
            if (typeof editorModel[this.name] === 'boolean') {
                this.type = 'boolean';
            }
            if (typeof editorModel[this.name] === 'number') {
                this.type = 'number';
            }
            if (editorModel[this.name] instanceof Date) {
                this.type = 'calendar';
            }
            // make an instance to read the properties
            this.label = editorModel["__displayName__" + this.name] || this.label || this.name;
            this.tooltip = editorModel["__displayDesc__" + this.name] || this.tooltip || this.name;
            // render as range id there is a range definition
            if (editorModel["__hasRangeFrom__" + this.name] && Number(editorModel["__hasRangeFrom__" + this.name])) {
                this.fromValue = editorModel["__hasRangeFrom__" + this.name];
                this.type = 'range';
            }
            if (editorModel["__hasRangeTo__" + this.name] && Number(editorModel["__hasRangeTo__" + this.name])) {
                this.toValue = editorModel["__hasRangeTo__" + this.name];
                this.type = 'range';
            }
            // placeholder
            if (editorModel["__hasWatermark__" + this.name]) {
                this.waterMark = editorModel["__watermark__" + this.name];
            }
            // templates
            if (editorModel["__hasTemplateHint__" + this.name]) {
                this.type = editorModel["__templateHint__" + this.name].toLowerCase();
                if (editorModel["__templateHintParams__" + this.name]) {
                    this.params = editorModel["__templateHintParams__" + this.name];
                }
            }
            // render hidden fields as hidden even in forms
            if (editorModel["__isHidden__" + this.name]) {
                this.type = 'hidden';
            }
            // check readonly
            if (editorModel["__isReadonly__" + this.name]) {
                this.readonly = !!editorModel["__isReadonly__" + this.name];
            }
        }
    };
    AcEditor.prototype.onValueChanged = function (data) {
        // check validation on change
        this.errors = new Array();
        for (var error in this.userForm.controls[this.name].errors) {
            this.errors.push(error);
        }
    };
    return AcEditor;
}());
__decorate$4([
    _angular_core.Input(),
    __metadata$4("design:type", String)
], exports.AcEditor.prototype, "name", void 0);
__decorate$4([
    _angular_core.Input(),
    __metadata$4("design:type", Object)
], exports.AcEditor.prototype, "type", void 0);
__decorate$4([
    _angular_core.Input(),
    __metadata$4("design:type", Object)
], exports.AcEditor.prototype, "labelDivider", void 0);
__decorate$4([
    _angular_core.Input(),
    __metadata$4("design:type", String)
], exports.AcEditor.prototype, "label", void 0);
__decorate$4([
    _angular_core.Input(),
    __metadata$4("design:type", String)
], exports.AcEditor.prototype, "tooltip", void 0);
__decorate$4([
    _angular_core.Input(),
    __metadata$4("design:type", _angular_forms.FormGroup)
], exports.AcEditor.prototype, "userForm", void 0);
__decorate$4([
    _angular_core.Input(),
    __metadata$4("design:type", Object)
], exports.AcEditor.prototype, "inline", void 0);
__decorate$4([
    _angular_core.Input(),
    __metadata$4("design:type", Object)
], exports.AcEditor.prototype, "enumValues", void 0);
__decorate$4([
    _angular_core.Input(),
    __metadata$4("design:type", Array)
], exports.AcEditor.prototype, "listValues", void 0);
__decorate$4([
    _angular_core.Input(),
    __metadata$4("design:type", Object)
], exports.AcEditor.prototype, "fromValue", void 0);
__decorate$4([
    _angular_core.Input(),
    __metadata$4("design:type", Object)
], exports.AcEditor.prototype, "toValue", void 0);
__decorate$4([
    _angular_core.Input(),
    __metadata$4("design:type", Object)
], exports.AcEditor.prototype, "waterMark", void 0);
__decorate$4([
    _angular_core.Input(),
    __metadata$4("design:type", Object)
], exports.AcEditor.prototype, "readonly", void 0);
__decorate$4([
    _angular_core.Output(),
    _angular_core.Input(),
    __metadata$4("design:type", Object)
], exports.AcEditor.prototype, "value", void 0);
exports.AcEditor = __decorate$4([
    _angular_core.Component({
        selector: 'ac-editor',
        styles: [
            'input[type="checkbox"] { display: none; }',
            'input[type="checkbox"] + label:before { font-family: FontAwesome; }',
            'input[type="checkbox"] + label:before { content: "\\f096"; }',
            'input[type="checkbox"]:checked + label:before { content: "\\f046"; }',
            "input[type=\"checkbox\"] + label { \n      display:inline-block; width:15px; height: 20px; margin: -1px 4px 0 0; vertical-align:middle; cursor: pointer; \n    }",
        ],
        template: "<div class=\"form-group row\" [formGroup]=\"userForm\" *ngIf=\"type != 'hidden'\" [ngClass]=\"{ 'has-danger': !userForm.controls[name].valid && userForm.controls[name].touched }\"><label class=\"col-form-label col-md-3 col-sm-10\" [attr.for]=\"name\" [attr.title]=\"tooltip\">{{ label }}{{ labelDivider }}</label><div [ngClass]=\"{ 'col-md-7 col-sm-10': inline }\"><textarea *ngIf=\"type == 'textarea'\" class=\"form-control\" [id]=\"name\" [readOnly]=\"readonly\" [formControlName]=\"name\" [attr.rows]=\"getParams('rows')\" [attr.cols]=\"getParams('cols')\"> </textarea><select *ngIf=\"type == 'enum' && enumValues\" class=\"form-control\" [id]=\"name\" [formControlName]=\"name\" [disabled]=\"readonly\"><option *ngFor=\"let option of enumValues\" [value]=\"option.key\">{{option.val}}</option></select><select *ngIf=\"type == 'list' && listValues\" class=\"form-control\" [id]=\"name\" [formControlName]=\"name\" [disabled]=\"readonly\"><option *ngFor=\"let option of listValues\" [value]=\"option.key\">{{option.val}}</option></select><div class=\"form-check\" *ngIf=\"type == 'radio' && (listValues || enumValues)\"><label class=\"form-check-label\" *ngIf=\"listValues\"><input type=\"radio\" *ngFor=\"let option of listValues\" [id]=\"name\" [formControlName]=\"name\" [disabled]=\"readonly\" [value]=\"option.key\">{{option.val}}</label><label class=\"form-check-label\" *ngIf=\"enumValues\"><input type=\"radio\" *ngFor=\"let option of enumValues\" [id]=\"name\" [formControlName]=\"name\" [disabled]=\"readonly\" [value]=\"option.key\">{{option.val}}</label></div><input *ngIf=\"type == 'range'\" type=\"range\" class=\"form-control\" [placeholder]=\"waterMark\" [attr.minvalue]=\"fromValue\" [attr.maxvalue]=\"toValue\" [attr.min]=\"fromValue\" [attr.max]=\"toValue\" [id]=\"name\" [formControlName]=\"name\" [title]=\"tooltip\"> <input *ngIf=\"type == 'calendar'\" type=\"date\" class=\"form-control\" [placeholder]=\"waterMark\" [readOnly]=\"readonly\" [id]=\"name\" [formControlName]=\"name\" [title]=\"tooltip\"> <input *ngIf=\"type == 'number'\" type=\"number\" class=\"form-control\" [placeholder]=\"waterMark\" [readOnly]=\"readonly\" [id]=\"name\" [formControlName]=\"name\" [title]=\"tooltip\"> <input *ngIf=\"type == 'boolean'\" type=\"checkbox\" class=\"form-control\" [readOnly]=\"readonly\" [id]=\"name\" [formControlName]=\"name\" [title]=\"tooltip\"><label *ngIf=\"type == 'boolean'\" [attr.for]=\"name\"></label><input *ngIf=\"type == 'text' || type == ''\" [placeholder]=\"waterMark\" [readOnly]=\"readonly\" type=\"text\" class=\"form-control\" [id]=\"name\" [formControlName]=\"name\"><ng-container *ngIf=\"type == 'template'\"><ng-content></ng-content></ng-container><span class=\"fa fa-warning text-danger form-control-feedback\" [hidden]=\"userForm.controls[name].valid || userForm.controls[name].pristine\"></span> <small class=\"text-danger\" [hidden]=\"userForm.controls[name].valid || userForm.controls[name].pristine\"><span *ngFor=\"let error of errors\">{{ userForm.controls[name].messages[error] }}</span></small></div></div><div [formGroup]=\"userForm\"><input *ngIf=\"type == 'hidden'\" [id]=\"name\" [formControlName]=\"name\" type=\"hidden\"></div>"
    }) //
    ,
    __metadata$4("design:paramtypes", [])
], exports.AcEditor);

var __decorate$5 = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d$$1;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d$$1 = decorators[i]) r = (c < 3 ? d$$1(r) : c > 3 ? d$$1(target, key, r) : d$$1(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata$5 = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
/**
 * This component creates a complete editor out of a viewmodel.
 * It creates one field for each property not decorated with @Hidden.
 *
 */
exports.AcAutoForm = (function () {
    function AcAutoForm() {
        /**
         * Ungrouped element will appear at the end, after all groupes.
         * If there are no groups this will be ignored. Optional.
         */
        this.ungroupedAfter = true;
        this.editors = new Array();
        this.groups = new Array();
    }
    AcAutoForm.prototype.ngOnInit = function () {
        // take all controls as is
        var hasDecorators = (this.formGroup['__editorModel__']);
        var _loop_1 = function () {
            var displayOrder = 0;
            var groupOrder = 0;
            var groupName;
            var groupDesc = void 0;
            var isInGroup = false;
            if (hasDecorators) {
                displayOrder = this_1.formGroup['__editorModel__']["__displayOrder__" + controlName];
                isInGroup = !!this_1.formGroup['__editorModel__']["__isGrouped__" + controlName];
                if (isInGroup) {
                    groupName = this_1.formGroup['__editorModel__']["__groupName__" + controlName];
                    groupOrder = this_1.formGroup['__editorModel__']["__groupOrder__" + controlName];
                    groupDesc = this_1.formGroup['__editorModel__']["__groupDesc__" + controlName];
                }
            }
            if (!isInGroup) {
                // if not part of any group just push to main group
                this_1.editors.push({
                    key: displayOrder,
                    editor: controlName
                });
            }
            else {
                // check if group already exists
                var existingGroupArray = this_1.groups.filter(function (g) { return g.name === groupName; });
                groupExists = existingGroupArray.length === 1;
                if (!groupExists) {
                    this_1.groups.push({
                        key: groupOrder,
                        name: groupName,
                        desc: groupDesc,
                        editors: new Array()
                    });
                }
                // add field to existing group (assume that here the group always exists)
                var existingGroup = this_1.groups.filter(function (g) { return g.name === groupName; })[0];
                // and store
                existingGroup.editors.push({
                    key: displayOrder,
                    editor: controlName
                });
            }
        };
        var this_1 = this, groupExists;
        // loop through all fields
        for (var controlName in this.formGroup.controls) {
            _loop_1();
        }
        // check the display decorator and sort
        this.editors.sort(function (e, n) { return e.key - n.key; });
        // sort groups
        this.groups.sort(function (e, n) { return e.key - n.key; });
    };
    AcAutoForm.prototype.grouped = function () {
        return this.groups && this.groups.length > 0;
    };
    return AcAutoForm;
}());
__decorate$5([
    _angular_core.Input(),
    __metadata$5("design:type", _angular_forms.FormGroup)
], exports.AcAutoForm.prototype, "formGroup", void 0);
__decorate$5([
    _angular_core.Input(),
    __metadata$5("design:type", Object)
], exports.AcAutoForm.prototype, "ungroupedAfter", void 0);
exports.AcAutoForm = __decorate$5([
    _angular_core.Component({
        selector: 'ac-autoform',
        styles: [
            'fieldset { border-top: 1px silver solid; padding: 10px; }',
            'legend { width: auto; padding-left: 10px; padding-right: 10px; font-size: 1em;}'
        ],
        template: "<ng-content></ng-content><ng-container *ngIf=\"!ungroupedAfter\"><ac-editor *ngFor=\"let editor of editors\" [name]=\"editor.editor\" [userForm]=\"formGroup\"></ac-editor></ng-container><ng-container *ngIf=\"grouped()\"><fieldset *ngFor=\"let group of groups\"><legend [attr.title]=\"group.desc\" *ngIf=\"group.name\">{{ group.name }}</legend><ac-editor *ngFor=\"let editor of group.editors\" [name]=\"editor.editor\" [userForm]=\"formGroup\"></ac-editor></fieldset></ng-container><ng-container *ngIf=\"ungroupedAfter\"><ac-editor *ngFor=\"let editor of editors\" [name]=\"editor.editor\" [userForm]=\"formGroup\"></ac-editor></ng-container>"
    }),
    __metadata$5("design:paramtypes", [])
], exports.AcAutoForm);

/**
 * The state of node
 */
/**
 * The state of node
 */ 
(function (AcTreeNodeState) {
    AcTreeNodeState[AcTreeNodeState["undefined"] = 0] = "undefined";
    AcTreeNodeState[AcTreeNodeState["checked"] = 1] = "checked";
    AcTreeNodeState[AcTreeNodeState["disabled"] = 2] = "disabled";
    AcTreeNodeState[AcTreeNodeState["expanded"] = 4] = "expanded";
    AcTreeNodeState[AcTreeNodeState["selected"] = 8] = "selected";
})(exports.AcTreeNodeState || (exports.AcTreeNodeState = {}));
/**
 * The view options of node
*/
var AcTreeNodeOptions = (function () {
    function AcTreeNodeOptions(options) {
        if (options) {
            // TODO: make type safe
            // for (let k in options) this[k] = options[k];
        }
    }
    return AcTreeNodeOptions;
}());

/**
 * The base class.
 */
var AcTreeNode = (function () {
    function AcTreeNode(options, nodes) {
        this.options = options || new AcTreeNodeOptions();
        if (nodes && nodes instanceof Array) {
            this.children = nodes;
        }
        else {
            if (nodes) {
                this.children = [nodes];
            }
        }
        this.stateChange = new _angular_core.EventEmitter();
        this.state = exports.AcTreeNodeState.undefined;
    }
    Object.defineProperty(AcTreeNode.prototype, "path", {
        get: function () {
            // walk up tree and return path with names
            var p = new Array();
            p.push(this);
            var n = this;
            while (this.parent) {
                n = n.parent;
                p.push(n);
            }
            return p;
        },
        enumerable: true,
        configurable: true
    });
    AcTreeNode.prototype.hasDirectAncestor = function (node) {
        return this.parent !== undefined;
    };
    Object.defineProperty(AcTreeNode.prototype, "hasChildren", {
        get: function () {
            return this.children !== undefined && this.children.length > 0;
        },
        enumerable: true,
        configurable: true
    });
    AcTreeNode.prototype.add = function (nodes) {
        if (nodes && nodes instanceof Array) {
            (_a = this.children).push.apply(_a, nodes);
        }
        else {
            this.children.push(nodes);
        }
        var _a;
    };
    AcTreeNode.prototype.remove = function (node) {
        var idx = this.children.indexOf(node);
        if (idx > -1) {
            this.children.splice(idx, 1);
            return true;
        }
        else {
            return false;
        }
    };
    Object.defineProperty(AcTreeNode.prototype, "state", {
        get: function () {
            return this._state;
        },
        set: function (value) {
            this._state = value;
            this.stateChange.emit(this._state);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AcTreeNode.prototype, "stateIsExpandend", {
        get: function () {
            return (this.state & exports.AcTreeNodeState.expanded) === exports.AcTreeNodeState.expanded;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AcTreeNode.prototype, "stateIsDisabled", {
        get: function () {
            return (this.state & exports.AcTreeNodeState.disabled) === exports.AcTreeNodeState.disabled;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AcTreeNode.prototype, "stateIsSelected", {
        get: function () {
            return (this.state & exports.AcTreeNodeState.selected) === exports.AcTreeNodeState.selected;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AcTreeNode.prototype, "stateIsChecked", {
        get: function () {
            return (this.state & exports.AcTreeNodeState.checked) === exports.AcTreeNodeState.checked;
        },
        enumerable: true,
        configurable: true
    });
    return AcTreeNode;
}());

var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d$$1, b) { d$$1.__proto__ = b; }) ||
        function (d$$1, b) { for (var p in b) if (b.hasOwnProperty(p)) d$$1[p] = b[p]; };
    return function (d$$1, b) {
        extendStatics(d$$1, b);
        function __() { this.constructor = d$$1; }
        d$$1.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
/**
 * A text node
 */
var AcTextTreeNode = (function (_super) {
    __extends(AcTextTreeNode, _super);
    function AcTextTreeNode(text, options, nodes) {
        var _this = _super.call(this, options, nodes) || this;
        _this.text = text;
        _this.name = text; // default, must be set explicitly
        return _this;
    }
    return AcTextTreeNode;
}(AcTreeNode));

var __extends$1 = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d$$1, b) { d$$1.__proto__ = b; }) ||
        function (d$$1, b) { for (var p in b) if (b.hasOwnProperty(p)) d$$1[p] = b[p]; };
    return function (d$$1, b) {
        extendStatics(d$$1, b);
        function __() { this.constructor = d$$1; }
        d$$1.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
/**
 * This type can load a component's UI into the node.
 */
var AcComponentTreeNode = (function (_super) {
    __extends$1(AcComponentTreeNode, _super);
    function AcComponentTreeNode(component, options, nodes) {
        var _this = _super.call(this, options, nodes) || this;
        _this.component = component;
        return _this;
    }
    return AcComponentTreeNode;
}(AcTreeNode));

var __decorate$6 = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d$$1;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d$$1 = decorators[i]) r = (c < 3 ? d$$1(r) : c > 3 ? d$$1(target, key, r) : d$$1(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata$6 = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
/**
 * A treeview that can handle @see AcTreeNode objects that create text, checkboxes, or highlights.
 * A node can have an icon. The icons are based on FontAwesome's css classes.
 */
exports.AcTreeView = (function () {
    function AcTreeView() {
        this.nodeClick = new _angular_core.EventEmitter();
        this.checkChanged = new _angular_core.EventEmitter();
        this.selectedChanged = new _angular_core.EventEmitter();
        this.collapseChanged = new _angular_core.EventEmitter();
    }
    AcTreeView.prototype.onNodeClick = function (node) {
        this.nodeClick.emit(node);
    };
    AcTreeView.prototype.onCheckChanged = function (node) {
        this.checkChanged.emit(node);
    };
    AcTreeView.prototype.onSelectedChanged = function (node) {
        this.selectedChanged.emit(node);
    };
    AcTreeView.prototype.onCollapseChanged = function (node) {
        this.collapseChanged.emit(node);
    };
    AcTreeView.prototype.ngOnChanges = function () {
        return;
    };
    return AcTreeView;
}());
__decorate$6([
    _angular_core.Input(),
    __metadata$6("design:type", AcTreeNode)
], exports.AcTreeView.prototype, "nodes", void 0);
__decorate$6([
    _angular_core.Output(),
    __metadata$6("design:type", _angular_core.EventEmitter)
], exports.AcTreeView.prototype, "nodeClick", void 0);
__decorate$6([
    _angular_core.Output(),
    __metadata$6("design:type", _angular_core.EventEmitter)
], exports.AcTreeView.prototype, "checkChanged", void 0);
__decorate$6([
    _angular_core.Output(),
    __metadata$6("design:type", _angular_core.EventEmitter)
], exports.AcTreeView.prototype, "selectedChanged", void 0);
__decorate$6([
    _angular_core.Output(),
    __metadata$6("design:type", _angular_core.EventEmitter)
], exports.AcTreeView.prototype, "collapseChanged", void 0);
exports.AcTreeView = __decorate$6([
    _angular_core.Component({
        selector: 'ac-tree',
        template: "<ul class=\"treeview\"><ac-treenode [node]=\"nodes\" (nodeClick)=\"onNodeClick($event)\" (checkChanged)=\"onCheckChanged($event)\" (selectedChanged)=\"onSelectedChanged($event)\" (collapseChanged)=\"onCollapseChanged($event)\"></ac-treenode></ul>",
        styles: ['.treeview { list-style: none; margin-left: -25px; }']
    }) //
    ,
    __metadata$6("design:paramtypes", [])
], exports.AcTreeView);

var __decorate$7 = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d$$1;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d$$1 = decorators[i]) r = (c < 3 ? d$$1(r) : c > 3 ? d$$1(target, key, r) : d$$1(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata$7 = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
/**
 * The treenode class represents a single treenode for the treeview. Actually, this renders the real UI.
 *
 * The component needs this input value:
 *
 * * @Input() node: AcTextTreeNode;
 *
 * The several options are set through the AcTreeNodeOptions object that is being held by the
 * AcTextTreeNode class. This includes colors, icons, and other styles.
 *
 * The component can fire these events:
 *
 * * @Output() nodeClick: EventEmitter<AcTreeNode>;
 * * @Output() checkChanged: EventEmitter<AcTreeNode>;
 * * @Output() selectedChanged: EventEmitter<AcTreeNode>;
 * * @Output() collapseChanged: EventEmitter<AcTreeNode>;
 *
 */
exports.AcTreeViewNode = AcTreeViewNode_1 = (function () {
    function AcTreeViewNode(el, renderer) {
        this.el = el;
        this.renderer = renderer;
        this.isExpanded = false;
        this.nodeClick = new _angular_core.EventEmitter();
        this.checkChanged = new _angular_core.EventEmitter();
        this.selectedChanged = new _angular_core.EventEmitter();
        this.collapseChanged = new _angular_core.EventEmitter();
    }
    AcTreeViewNode.prototype.ngOnInit = function () {
        var _this = this;
        // set HTML according to options
        this.collapseClasses = new Array();
        this.iconClasses = new Array();
        // expander area with icon
        if (this.node) {
            // expect a font-awesome class with or without fa class
            if (this.node.options && this.node.options.icon) {
                if (this.node.options.icon.indexOf(AcTreeViewNode_1.pfxIcon + " ") !== 0) {
                    this.iconClasses.push('fa');
                }
                this.iconClasses.push(this.node.options.icon);
            }
            if (this.node.options && this.node.options.href) {
                this.href = this.node.options.href;
            }
            else {
                this.href = '';
            }
            // open/close area for elements with children
            if (this.node.hasChildren) {
                this.collapseClasses.push('ac-icon'); // base class
                this.collapseClasses.push(AcTreeViewNode_1.pfxIcon);
                this.collapseClasses.push(AcTreeViewNode_1.opnIcon);
                // collapsed by default
                this.node.state &= ~exports.AcTreeNodeState.checked;
            }
            if (this.node.options && this.node.options.color) {
                this.foreColor = this.node.options.color;
            }
            if (this.node.options && this.node.options.backColor) {
                this.backColor = this.node.options.backColor;
            }
            this.node.stateChange.subscribe(function (se) {
                _this.collapseChanged.emit(_this.node);
                if (!_this.node.stateIsExpandend) {
                    _this.collapseClasses = _this.collapseClasses.filter(function (s) { return s != AcTreeViewNode_1.clsIcon; });
                    _this.collapseClasses.push(AcTreeViewNode_1.opnIcon);
                }
                else {
                    _this.collapseClasses = _this.collapseClasses.filter(function (s) { return s != AcTreeViewNode_1.opnIcon; });
                    _this.collapseClasses.push(AcTreeViewNode_1.clsIcon);
                }
                _this.isExpanded = _this.node.stateIsExpandend;
            });
        }
    };
    // forward events in the node tree
    AcTreeViewNode.prototype.onNodeClick = function (node) {
        this.nodeClick.emit(node);
    };
    AcTreeViewNode.prototype.onCheckChanged = function (node) {
        this.checkChanged.emit(node);
    };
    AcTreeViewNode.prototype.onSelectedChanged = function (node) {
        this.selectedChanged.emit(node);
    };
    AcTreeViewNode.prototype.onCollapseChanged = function (node) {
        if (this.node.options.collapsable && this.node.hasChildren) {
            this.collapseChanged.emit(node);
        }
    };
    AcTreeViewNode.prototype.handleCheckChange = function () {
        if (this.node.options.checkable && !this.node.stateIsDisabled) {
            if (this.node.state & exports.AcTreeNodeState.checked) {
                this.node.state &= ~exports.AcTreeNodeState.checked;
            }
            else {
                this.node.state |= exports.AcTreeNodeState.checked;
            }
            this.checkChanged.emit(this.node);
        }
    };
    AcTreeViewNode.prototype.handleClick = function ($event) {
        $event.stopPropagation();
        // always emit click
        this.nodeClick.emit(this.node);
    };
    AcTreeViewNode.prototype.handleCollapse = function () {
        // if collapsable handle icons and view state
        if (this.node.options.collapsable) {
            // toggle state
            if (this.node.state && (this.node.state & exports.AcTreeNodeState.expanded)) {
                this.node.state &= ~exports.AcTreeNodeState.expanded;
            }
            else {
                this.node.state |= exports.AcTreeNodeState.expanded;
            }
        }
    };
    AcTreeViewNode.prototype.handlePreSelection = function (state) {
        // handle the states according to options
        if (this.node.options.selectable && !this.node.stateIsDisabled) {
            if (state) {
                if (this.node.options && this.node.options.color) {
                    this.foreColor = this.node.options.backColor;
                }
                else {
                    this.foreColor = '#fff';
                }
                if (this.node.options && this.node.options.backColor) {
                    this.backColor = this.node.options.color;
                }
                else {
                    this.backColor = '#000';
                }
            }
            else {
                if (this.node.options && this.node.options.color) {
                    this.foreColor = this.node.options.color;
                }
                else {
                    this.foreColor = '#000';
                }
                if (this.node.options && this.node.options.backColor) {
                    this.backColor = this.node.options.backColor;
                }
                else {
                    this.backColor = '#fff';
                }
            }
            this.preSelectState = state;
        }
    };
    AcTreeViewNode.prototype.handleSelection = function ($event) {
        // handle the states according to options
        if (this.node.options.selectable && !this.node.stateIsDisabled && this.preSelectState === true) {
            if (this.node.state & exports.AcTreeNodeState.selected) {
                this.node.state &= ~exports.AcTreeNodeState.selected;
            }
            else {
                this.node.state |= exports.AcTreeNodeState.selected;
            }
            if (this.node.stateIsSelected) {
                if (this.node.options && this.node.options.color) {
                    this.foreColor = this.node.options.backColor;
                }
                else {
                    this.foreColor = '#fff';
                }
                if (this.node.options && this.node.options.backColor) {
                    this.backColor = this.node.options.color;
                }
                else {
                    this.backColor = '#000';
                }
            }
            else {
                if (this.node.options && this.node.options.color) {
                    this.foreColor = this.node.options.color;
                }
                else {
                    this.foreColor = '#000';
                }
                if (this.node.options && this.node.options.backColor) {
                    this.backColor = this.node.options.backColor;
                }
                else {
                    this.backColor = '#fff';
                }
            }
            this.selectedChanged.emit(this.node);
        }
        if (!this.href) {
            $event.preventDefault();
            $event.stopPropagation();
        }
    };
    return AcTreeViewNode;
}());
exports.AcTreeViewNode.pfxIcon = 'fa';
exports.AcTreeViewNode.opnIcon = 'fa-plus';
exports.AcTreeViewNode.clsIcon = 'fa-minus';
__decorate$7([
    _angular_core.Input(),
    __metadata$7("design:type", AcTextTreeNode)
], exports.AcTreeViewNode.prototype, "node", void 0);
__decorate$7([
    _angular_core.Output(),
    __metadata$7("design:type", _angular_core.EventEmitter)
], exports.AcTreeViewNode.prototype, "nodeClick", void 0);
__decorate$7([
    _angular_core.Output(),
    __metadata$7("design:type", _angular_core.EventEmitter)
], exports.AcTreeViewNode.prototype, "checkChanged", void 0);
__decorate$7([
    _angular_core.Output(),
    __metadata$7("design:type", _angular_core.EventEmitter)
], exports.AcTreeViewNode.prototype, "selectedChanged", void 0);
__decorate$7([
    _angular_core.Output(),
    __metadata$7("design:type", _angular_core.EventEmitter)
], exports.AcTreeViewNode.prototype, "collapseChanged", void 0);
exports.AcTreeViewNode = AcTreeViewNode_1 = __decorate$7([
    _angular_core.Component({
        selector: 'ac-treenode',
        template: "<li class=\"treeview\" (click)=\"handleClick($event)\"><i class=\"ac-collapse\" [ngClass]=\"collapseClasses\" *ngIf=\"node.hasChildren\" (click)=\"handleCollapse()\"></i> <i class=\"ac-collapse\" *ngIf=\"!node.hasChildren\"></i> <i class=\"ac-icon\" [ngClass]=\"iconClasses\" [style.color]=\"node.options.iconColor\" *ngIf=\"!node.options.checkable\"></i> <input type=\"checkbox\" [id]=\"node.name\" *ngIf=\"node.options.checkable\" [checked]=\"node.stateIsChecked\" (click)=\"handleCheckChange()\"><label [attr.for]=\"node.name\"></label><a class=\"ac-container\" [href]=\"href\" [style.color]=\"foreColor\" [style.background-color]=\"backColor\" (mouseover)=\"handlePreSelection(true)\" (mouseout)=\"handlePreSelection(false)\" (click)=\"handleSelection($event)\">{{ node.text }}</a><ul class=\"treeview\" *ngIf=\"node.hasChildren\" [hidden]=\"!isExpanded\"><ac-treenode *ngFor=\"let child of node.children\" [node]=\"child\" (nodeClick)=\"onNodeClick($event)\" (checkChanged)=\"onCheckChanged($event)\" (selectedChanged)=\"onSelectedChanged($event)\" (collapseChanged)=\"onCollapseChanged($event)\"></ac-treenode></ul></li>",
        styles: ["ul.treeview{list-style:none;margin-left:-2em}li.treeview{margin-left:10px;margin-bottom:3px;box-sizing:border-box}li.treeview a.ac-container{border-radius:2px;display:inline-block;padding:3px;text-decoration:none}li.treeview input[type=checkbox]{display:none}li.treeview input[type=checkbox]+label:before{font-family:FontAwesome}li.treeview input[type=checkbox]+label:before{content:\"\\f096\"}li.treeview input[type=checkbox]:checked+label:before{content:\"\\f046\"}li.treeview input[type=checkbox]+label{display:inline-block;width:15px;height:20px;margin:-1px 4px 0 0;vertical-align:middle;cursor:pointer}li.treeview i.ac-collapse{width:15px;cursor:pointer;display:inline-block;margin-left:-1.7em}li.treeview i.ac-icon{width:15px;cursor:pointer;display:inline-block}li.treeview .ac-node-disabled{color:silver;cursor:not-allowed}"]
    }),
    __metadata$7("design:paramtypes", [_angular_core.ElementRef, _angular_core.Renderer])
], exports.AcTreeViewNode);
var AcTreeViewNode_1;

var __decorate$8 = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d$$1;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d$$1 = decorators[i]) r = (c < 3 ? d$$1(r) : c > 3 ? d$$1(target, key, r) : d$$1(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata$8 = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
/**
 * A simple round analog clock based on SVG and JavaScript for animation.
 */
exports.AcAnalogClock = (function () {
    function AcAnalogClock(rd, document) {
        this.rd = rd;
        this.document = document;
        /**
         * The color of the second arrow.
         */
        this.secondColor = '#C1EFED';
        /**
         * The color of the minute arrow.
         */
        this.minuteColor = '#fdfdfd';
        /**
         * The color of the hour arrow.
         */
        this.hourColor = '#fffbf9';
        this.minuteClock = new _angular_core.EventEmitter();
        this.hourClock = new _angular_core.EventEmitter();
        this.bgColor = '#92345';
        this.diameter = 100;
    }
    AcAnalogClock.prototype.ngOnChanges = function (changes) {
        if (changes['diameter']) {
            this.makeMarks();
        }
    };
    AcAnalogClock.prototype.ngAfterViewInit = function () {
        var _this = this;
        this.makeMarks();
        // create animation string
        var shifter = function (val) { return [val, _this.diameter, _this.diameter].join(' '); };
        // // from real time
        var date = new Date();
        var secAngle = 360 * date.getSeconds() / 60;
        var minuteAngle = 360 * date.getMinutes() / 60;
        var hoursAngle = 360 * date.getHours() / 12 + date.getMinutes() / 2;
        // // assign animation flow
        this.setRotate(this.secondhandTransform, shifter(secAngle));
        this.setRotate(this.minutehandTransform, shifter(minuteAngle));
        this.setRotate(this.hourhandTransform, shifter(hoursAngle));
        setInterval(function () {
            date = new Date();
            secAngle = 360 * date.getSeconds() / 60;
            minuteAngle = 360 * date.getMinutes() / 60;
            hoursAngle = 360 * date.getHours() / 12 + date.getMinutes() / 2;
            _this.setRotate(_this.secondhandTransform, shifter(secAngle));
            _this.setRotate(_this.minutehandTransform, shifter(minuteAngle));
            _this.setRotate(_this.hourhandTransform, shifter(hoursAngle));
        }, 1000);
    };
    AcAnalogClock.prototype.setRotate = function (element, value) {
        element.nativeElement.setAttribute('transform', "rotate(" + value + ")");
    };
    AcAnalogClock.prototype.makeMarks = function () {
        var marks = this.svg.nativeElement.querySelectorAll('line[mark]');
        if (marks && marks.length > 0) {
            for (var i_1 = 0; i_1 < marks.length; i_1++) {
                marks[i_1].remove();
            }
        }
        // make hour marks
        for (var i = 1; i <= 12; i++) {
            var el = this.document.createElementNS('http://www.w3.org/2000/svg', 'line');
            el.setAttribute('x1', this.diameter.toString());
            el.setAttribute('y1', (this.diameter * 0.3).toString());
            el.setAttribute('x2', this.diameter.toString());
            el.setAttribute('y2', (this.diameter * 0.4).toString());
            el.setAttribute('transform', 'rotate(' + (i * 360 / 12) + (" " + this.diameter + " " + this.diameter + ")"));
            el.setAttribute('style', 'stroke: #ffffff;');
            el.setAttribute('mark', '');
            this.svg.nativeElement.appendChild(el);
        }
    };
    return AcAnalogClock;
}());
__decorate$8([
    _angular_core.Input(),
    __metadata$8("design:type", String)
], exports.AcAnalogClock.prototype, "bgColor", void 0);
__decorate$8([
    _angular_core.Input(),
    __metadata$8("design:type", Object)
], exports.AcAnalogClock.prototype, "secondColor", void 0);
__decorate$8([
    _angular_core.Input(),
    __metadata$8("design:type", Object)
], exports.AcAnalogClock.prototype, "minuteColor", void 0);
__decorate$8([
    _angular_core.Input(),
    __metadata$8("design:type", Object)
], exports.AcAnalogClock.prototype, "hourColor", void 0);
__decorate$8([
    _angular_core.Input(),
    __metadata$8("design:type", Number)
], exports.AcAnalogClock.prototype, "diameter", void 0);
__decorate$8([
    _angular_core.Input(),
    __metadata$8("design:type", Boolean)
], exports.AcAnalogClock.prototype, "showSeconds", void 0);
__decorate$8([
    _angular_core.Output(),
    __metadata$8("design:type", _angular_core.EventEmitter)
], exports.AcAnalogClock.prototype, "minuteClock", void 0);
__decorate$8([
    _angular_core.Output(),
    __metadata$8("design:type", _angular_core.EventEmitter)
], exports.AcAnalogClock.prototype, "hourClock", void 0);
__decorate$8([
    _angular_core.ViewChild('hourhandTransform'),
    __metadata$8("design:type", _angular_core.ElementRef)
], exports.AcAnalogClock.prototype, "hourhandTransform", void 0);
__decorate$8([
    _angular_core.ViewChild('minutehandTransform'),
    __metadata$8("design:type", _angular_core.ElementRef)
], exports.AcAnalogClock.prototype, "minutehandTransform", void 0);
__decorate$8([
    _angular_core.ViewChild('secondhandTransform'),
    __metadata$8("design:type", _angular_core.ElementRef)
], exports.AcAnalogClock.prototype, "secondhandTransform", void 0);
__decorate$8([
    _angular_core.ViewChild('svg'),
    __metadata$8("design:type", _angular_core.ElementRef)
], exports.AcAnalogClock.prototype, "svg", void 0);
exports.AcAnalogClock = __decorate$8([
    _angular_core.Component({
        selector: 'ac-analogclock',
        styles: ["svg { display: block; }"],
        template: "<svg xmlns=\"http://www.w3.org/2000/svg\" version=\"1.1\" [attr.width]=\"diameter * 2\" [attr.height]=\"diameter * 2\" #svg><filter id=\"innerShadow\" x=\"-20%\" y=\"-20%\" width=\"140%\" height=\"140%\"><feGaussianBlur in=\"SourceGraphic\" stdDeviation=\"3\" result=\"blur\"/><feOffset in=\"blur\" dx=\"2.5\" dy=\"2.5\"/></filter><g><circle id=\"shadow\" style=\"fill:rgba(0,0,0,0.1)\" [attr.cx]=\"diameter - 3\" [attr.cy]=\"diameter\" [attr.r]=\"diameter - 3\" filter=\"url(#innerShadow)\"></circle><circle id=\"circle\" style=\"stroke: #FFF; stroke-width: 12px;\" [style.stroke-width]=\"diameter * 0.12\" [attr.cx]=\"diameter\" [attr.cy]=\"diameter\" [attr.fill]=\"bgColor\" [attr.r]=\"diameter - (diameter * 0.2)\"></circle></g><g><line [attr.x1]=\"diameter\" [attr.y1]=\"diameter\" [attr.x2]=\"diameter\" [attr.y2]=\"diameter * 0.55\" transform=\"rotate(90 100 100)\" style=\"stroke-width: 4px; stroke: #fffbf9;\" #hourhandTransform [style.stroke]=\"hourColor\"></line><line [attr.x1]=\"diameter\" [attr.y1]=\"diameter\" [attr.x2]=\"diameter\" [attr.y2]=\"diameter * 0.40\" transform=\"rotate(180 100 100)\" style=\"stroke-width: 3px; stroke: #fdfdfd;\" #minutehandTransform [style.stroke]=\"minuteColor\"></line><line [attr.x1]=\"diameter\" [attr.y1]=\"diameter\" [attr.x2]=\"diameter\" [attr.y2]=\"diameter * 0.30\" transform=\"rotate(270 100 100)\" style=\"stroke-width: 2px; stroke: #C1EFED;\" #secondhandTransform [style.stroke]=\"secondColor\"></line></g><circle id=\"center\" style=\"fill:#128A86; stroke: #C1EFED; stroke-width: 2px;\" [attr.cx]=\"diameter\" [attr.cy]=\"diameter\" r=\"3\"></circle></svg>"
    }),
    __param(1, _angular_core.Inject(_angular_platformBrowser.DOCUMENT)),
    __metadata$8("design:paramtypes", [_angular_core.Renderer, HTMLDocument])
], exports.AcAnalogClock);

/**
 * Describe a header field with name and tooltip.
 */
var AcDataGridHeader = (function () {
    /**
     * The ctor
     * @param text: The text to display.
     * @param desc: A tooltip that is shown on mouseover (using the `title` attribute).
     * @param prop: The propertie's internal name.
     */
    function AcDataGridHeader(text, desc, prop, hidden) {
        if (hidden === void 0) { hidden = false; }
        this.text = text;
        this.desc = desc;
        this.prop = prop;
        this.hidden = hidden;
    }
    return AcDataGridHeader;
}());

var __decorate$9 = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d$$1;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d$$1 = decorators[i]) r = (c < 3 ? d$$1(r) : c > 3 ? d$$1(target, key, r) : d$$1(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata$9 = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var SVOGV_COMPONENTS = [
    exports.AcInfoBox,
    exports.AcDataGridPagination,
    exports.AcTreeView,
    exports.AcTreeViewNode,
    exports.AcEditor,
    exports.AcAutoForm,
    exports.AcLoaderIcon,
    exports.AcAnalogClock,
    exports.AcFormatDataPipe
];
var SVOGV_EXPORTS = [
    [d.Compare, d.Display, d.DisplayGroup, d.Email, d.Format, d.Range, d.Readonly, d.Required,
        d.Hidden, d.MaxLength, d.MinLength, d.StringLength, d.Pattern, d.TemplateHint]
];
/**
 * The actual SVOGV Module definition using the root module.
 */
exports.SvogvModule = SvogvModule_1 = (function () {
    function SvogvModule(injector) {
        d.SvogvCoreModule.forRoot();
        SvogvModule_1.injector = injector;
    }
    return SvogvModule;
}());
exports.SvogvModule = SvogvModule_1 = __decorate$9([
    _angular_core.NgModule({
        imports: [_angular_platformBrowser.BrowserModule, _angular_router.RouterModule, _angular_forms.ReactiveFormsModule, _angular_forms.FormsModule, d.SvogvCoreModule],
        declarations: SVOGV_COMPONENTS,
        providers: [d.FormValidatorService, d.FormValidatorFromJsonService],
        exports: SVOGV_COMPONENTS.concat(SVOGV_EXPORTS)
    }),
    __metadata$9("design:paramtypes", [_angular_core.Injector])
], exports.SvogvModule);
var SvogvModule_1;

/**
 * The item with data and meta data.
 */
var AcDataGridItem = (function () {
    /**
     * The ctor
     * @param value: The value to show.
     * @param prop: The name of the property.
     * @param pipe: An attached pipe, if any. Used for dynamic formatting
     */
    function AcDataGridItem() {
    }
    AcDataGridItem.prototype.getFormatted = function (pipeArgs) {
        var pipe = exports.SvogvModule.injector.get(this.pipeToken);
        return pipe.transform.apply(pipe, [this.value].concat(pipeArgs));
    };
    return AcDataGridItem;
}());

(function (Direction) {
    Direction[Direction["Ascending"] = 0] = "Ascending";
    Direction[Direction["Descending"] = 1] = "Descending";
})(exports.Direction || (exports.Direction = {}));
/**
 * The controlling class for Grid applications.
 *
 * This class takes an array of elements and handles:
 * - visible headers, managed by @Hidden() decorator
 * - create header titles, managed by @Display() decorator
 * - sorting
 * - filtering
 * - count total rows
 * - paging
 */
var AcDataGridModel = (function () {
    function AcDataGridModel(items, typeInstance, pageSize) {
        if (pageSize === void 0) { pageSize = 10; }
        this.searchValue = {};
        this.currentPageIndex = 1;
        this._items = items;
        this.pageSize = pageSize;
        if (typeInstance) {
            // make header from decorators, omit if null
            this.createHeadersForType(typeInstance);
        }
    }
    Object.defineProperty(AcDataGridModel.prototype, "totalRows", {
        get: function () {
            return this._items.length;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AcDataGridModel.prototype, "totalFilteredRows", {
        get: function () {
            return (this.itemsFiltered) ? this.itemsFiltered.length : 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AcDataGridModel.prototype, "currentRowStart", {
        get: function () {
            return this.totalRows > this.pageSize
                ? (this.startRow + 1)
                : this.totalRows === 0 ? 0 : 1;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AcDataGridModel.prototype, "currentRowEnd", {
        get: function () {
            return (this.startRow + this.pageSize) < this.totalRows
                ? (this.startRow + this.pageSize)
                : this.totalRows;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AcDataGridModel.prototype, "startRow", {
        get: function () {
            if (this.currentPageIndex === 0) {
                return 0;
            }
            return (this.currentPageIndex - 1) * this.pageSize;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AcDataGridModel.prototype, "maxPageIndex", {
        get: function () {
            var index = Math.ceil(this.totalFilteredRows / this.pageSize);
            return index;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AcDataGridModel.prototype, "items", {
        get: function () {
            return this._items;
        },
        set: function (value) {
            this._items = value;
        },
        enumerable: true,
        configurable: true
    });
    AcDataGridModel.prototype.getItemSorted = function (sortColumn, sortDirection) {
        if (sortDirection === exports.Direction.Ascending) {
            return this.items.sort(function (a, b) { return a[sortColumn] > b[sortColumn] ? 1 : -1; });
        }
        else {
            return this.items.sort(function (a, b) { return a[sortColumn] < b[sortColumn] ? 1 : -1; });
        }
    };
    Object.defineProperty(AcDataGridModel.prototype, "itemsFiltered", {
        get: function () {
            var _this = this;
            return this.items.filter(function (item) { return Object.same(_this.searchValue, item); });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AcDataGridModel.prototype, "itemsOnCurrentPage", {
        get: function () {
            return this.itemsFiltered.slice(this.startRow, this.startRow + this.pageSize);
        },
        enumerable: true,
        configurable: true
    });
    // The view can get col by col filtered for valid headers
    AcDataGridModel.prototype.columnsOfItemValues = function (item) {
        // we return all if no headers
        var columns = new Array();
        if (!this._headers) {
            for (var prop in item) {
                columns.push(item[prop]);
            }
        }
        else {
            this.headers
                .forEach(function (e, idx) { return columns.push(item[e.prop]); });
        }
        return columns;
    };
    AcDataGridModel.prototype.columnsOfItem = function (item) {
        // we return all if no headers
        var columns = new Array();
        var currentHeaders = this.headers || Object.keys(item).map(function (h) { return new AcDataGridHeader(h, null, h, false); });
        currentHeaders
            .forEach(function (h, idx) {
            var e = new AcDataGridItem();
            e.value = item[h.prop];
            e.prop = h.prop;
            var hasPipe = e["__hasPipe__" + h.prop];
            if (hasPipe) {
                e.pipeToken = hasPipe;
            }
            columns.push(e);
        });
        return columns;
    };
    AcDataGridModel.prototype.sortColumn = function (colName, dir) {
        this.items.sort(function (a, b) { return dir === 'desc' ? (a[colName] > b[colName] ? 1 : -1) : (a[colName] > b[colName] ? -1 : 1); });
    };
    Object.defineProperty(AcDataGridModel.prototype, "headers", {
        get: function () {
            return this._headers.filter(function (h) { return !h.hidden; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AcDataGridModel.prototype, "headersNotVisible", {
        get: function () {
            return this._headers.filter(function (h) { return h.hidden; });
        },
        enumerable: true,
        configurable: true
    });
    AcDataGridModel.prototype.removeColumn = function (colname) {
        var col = this._headers.find(function (h) { return h.prop === colname; });
        if (col) {
            col.hidden = true;
        }
    };
    AcDataGridModel.prototype.addColumn = function (colname) {
        var col = this._headers.find(function (h) { return h.prop === colname; });
        if (col) {
            col.hidden = false;
        }
    };
    AcDataGridModel.prototype.createHeadersForType = function (type) {
        // assume simple object structure, iterating an array of viewmodels
        // has at least one row, so we can read the headers
        // first we read the properties
        this._headers = new Array();
        for (var p in type) {
            // either propname or decorator name
            var propName = type["__displayName__" + p] || p;
            var propDesc = type["__displayDesc__" + p] || p;
            // check if hidden, show if no hidden decorator
            var isHidden = type["__isHidden__" + p] || false;
            this._headers.push(new AcDataGridHeader(propName, propDesc, p, isHidden));
        }
    };
    return AcDataGridModel;
}());

exports.AcInfoBoxOptions = AcInfoBoxOptions;
exports.AcDataGridModel = AcDataGridModel;
exports.AcDataGridHeader = AcDataGridHeader;
exports.AcTreeNodeOptions = AcTreeNodeOptions;
exports.AcTreeNode = AcTreeNode;
exports.AcTextTreeNode = AcTextTreeNode;
exports.AcComponentTreeNode = AcComponentTreeNode;

Object.defineProperty(exports, '__esModule', { value: true });

})));
