var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
import { Component, Inject, Input, Output, EventEmitter, ViewChild, ElementRef, Renderer } from '@angular/core';
import { DOCUMENT } from '@angular/platform-browser';
/**
 * A simple round analog clock based on SVG and JavaScript for animation.
 */
var AcAnalogClock = (function () {
    function AcAnalogClock(rd, document) {
        this.rd = rd;
        this.document = document;
        /**
         * The color of the second arrow.
         */
        this.secondColor = '#C1EFED';
        /**
         * The color of the minute arrow.
         */
        this.minuteColor = '#fdfdfd';
        /**
         * The color of the hour arrow.
         */
        this.hourColor = '#fffbf9';
        this.minuteClock = new EventEmitter();
        this.hourClock = new EventEmitter();
        this.bgColor = '#92345';
        this.diameter = 100;
    }
    AcAnalogClock.prototype.ngOnChanges = function (changes) {
        if (changes['diameter']) {
            this.makeMarks();
        }
    };
    AcAnalogClock.prototype.ngAfterViewInit = function () {
        var _this = this;
        this.makeMarks();
        // create animation string
        var shifter = function (val) { return [val, _this.diameter, _this.diameter].join(' '); };
        // // from real time
        var date = new Date();
        var secAngle = 360 * date.getSeconds() / 60;
        var minuteAngle = 360 * date.getMinutes() / 60;
        var hoursAngle = 360 * date.getHours() / 12 + date.getMinutes() / 2;
        // // assign animation flow
        this.setRotate(this.secondhandTransform, shifter(secAngle));
        this.setRotate(this.minutehandTransform, shifter(minuteAngle));
        this.setRotate(this.hourhandTransform, shifter(hoursAngle));
        setInterval(function () {
            date = new Date();
            secAngle = 360 * date.getSeconds() / 60;
            minuteAngle = 360 * date.getMinutes() / 60;
            hoursAngle = 360 * date.getHours() / 12 + date.getMinutes() / 2;
            _this.setRotate(_this.secondhandTransform, shifter(secAngle));
            _this.setRotate(_this.minutehandTransform, shifter(minuteAngle));
            _this.setRotate(_this.hourhandTransform, shifter(hoursAngle));
        }, 1000);
    };
    AcAnalogClock.prototype.setRotate = function (element, value) {
        element.nativeElement.setAttribute('transform', "rotate(" + value + ")");
    };
    AcAnalogClock.prototype.makeMarks = function () {
        var marks = this.svg.nativeElement.querySelectorAll('line[mark]');
        if (marks && marks.length > 0) {
            for (var i_1 = 0; i_1 < marks.length; i_1++) {
                marks[i_1].remove();
            }
        }
        // make hour marks
        for (var i = 1; i <= 12; i++) {
            var el = this.document.createElementNS('http://www.w3.org/2000/svg', 'line');
            el.setAttribute('x1', this.diameter.toString());
            el.setAttribute('y1', (this.diameter * 0.3).toString());
            el.setAttribute('x2', this.diameter.toString());
            el.setAttribute('y2', (this.diameter * 0.4).toString());
            el.setAttribute('transform', 'rotate(' + (i * 360 / 12) + (" " + this.diameter + " " + this.diameter + ")"));
            el.setAttribute('style', 'stroke: #ffffff;');
            el.setAttribute('mark', '');
            this.svg.nativeElement.appendChild(el);
        }
    };
    return AcAnalogClock;
}());
__decorate([
    Input(),
    __metadata("design:type", String)
], AcAnalogClock.prototype, "bgColor", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], AcAnalogClock.prototype, "secondColor", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], AcAnalogClock.prototype, "minuteColor", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], AcAnalogClock.prototype, "hourColor", void 0);
__decorate([
    Input(),
    __metadata("design:type", Number)
], AcAnalogClock.prototype, "diameter", void 0);
__decorate([
    Input(),
    __metadata("design:type", Boolean)
], AcAnalogClock.prototype, "showSeconds", void 0);
__decorate([
    Output(),
    __metadata("design:type", EventEmitter)
], AcAnalogClock.prototype, "minuteClock", void 0);
__decorate([
    Output(),
    __metadata("design:type", EventEmitter)
], AcAnalogClock.prototype, "hourClock", void 0);
__decorate([
    ViewChild('hourhandTransform'),
    __metadata("design:type", ElementRef)
], AcAnalogClock.prototype, "hourhandTransform", void 0);
__decorate([
    ViewChild('minutehandTransform'),
    __metadata("design:type", ElementRef)
], AcAnalogClock.prototype, "minutehandTransform", void 0);
__decorate([
    ViewChild('secondhandTransform'),
    __metadata("design:type", ElementRef)
], AcAnalogClock.prototype, "secondhandTransform", void 0);
__decorate([
    ViewChild('svg'),
    __metadata("design:type", ElementRef)
], AcAnalogClock.prototype, "svg", void 0);
AcAnalogClock = __decorate([
    Component({
        selector: 'ac-analogclock',
        styles: ["svg { display: block; }"],
        template: "<svg xmlns=\"http://www.w3.org/2000/svg\" version=\"1.1\" [attr.width]=\"diameter * 2\" [attr.height]=\"diameter * 2\" #svg><filter id=\"innerShadow\" x=\"-20%\" y=\"-20%\" width=\"140%\" height=\"140%\"><feGaussianBlur in=\"SourceGraphic\" stdDeviation=\"3\" result=\"blur\"/><feOffset in=\"blur\" dx=\"2.5\" dy=\"2.5\"/></filter><g><circle id=\"shadow\" style=\"fill:rgba(0,0,0,0.1)\" [attr.cx]=\"diameter - 3\" [attr.cy]=\"diameter\" [attr.r]=\"diameter - 3\" filter=\"url(#innerShadow)\"></circle><circle id=\"circle\" style=\"stroke: #FFF; stroke-width: 12px;\" [style.stroke-width]=\"diameter * 0.12\" [attr.cx]=\"diameter\" [attr.cy]=\"diameter\" [attr.fill]=\"bgColor\" [attr.r]=\"diameter - (diameter * 0.2)\"></circle></g><g><line [attr.x1]=\"diameter\" [attr.y1]=\"diameter\" [attr.x2]=\"diameter\" [attr.y2]=\"diameter * 0.55\" transform=\"rotate(90 100 100)\" style=\"stroke-width: 4px; stroke: #fffbf9;\" #hourhandTransform [style.stroke]=\"hourColor\"></line><line [attr.x1]=\"diameter\" [attr.y1]=\"diameter\" [attr.x2]=\"diameter\" [attr.y2]=\"diameter * 0.40\" transform=\"rotate(180 100 100)\" style=\"stroke-width: 3px; stroke: #fdfdfd;\" #minutehandTransform [style.stroke]=\"minuteColor\"></line><line [attr.x1]=\"diameter\" [attr.y1]=\"diameter\" [attr.x2]=\"diameter\" [attr.y2]=\"diameter * 0.30\" transform=\"rotate(270 100 100)\" style=\"stroke-width: 2px; stroke: #C1EFED;\" #secondhandTransform [style.stroke]=\"secondColor\"></line></g><circle id=\"center\" style=\"fill:#128A86; stroke: #C1EFED; stroke-width: 2px;\" [attr.cx]=\"diameter\" [attr.cy]=\"diameter\" r=\"3\"></circle></svg>"
    }),
    __param(1, Inject(DOCUMENT)),
    __metadata("design:paramtypes", [Renderer, HTMLDocument])
], AcAnalogClock);
export { AcAnalogClock };

//# sourceMappingURL=ac-analogclock.component.js.map
