var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Component, Input, Output, EventEmitter } from '@angular/core';
/**
 * The pagination component creates a few buttons to navigate a grid. The underlaying model
 * is going to handle the date on the client. The pagination does not support a server backend,
 * all relevant data must be loaded first.
 */
var AcDataGridPagination = (function () {
    function AcDataGridPagination() {
        /**
         * An event fired once the user has changed the page by clicking a button.
         */
        this.pageNumberChanged = new EventEmitter();
        this.currentPageNumber = 1;
    }
    AcDataGridPagination.prototype.ngOnInit = function () {
        this.setCurrentPage(1);
    };
    AcDataGridPagination.prototype.ngOnChanges = function (changes) {
        var _this = this;
        if (changes['maxPageIndex']) {
            var change = changes['maxPageIndex'];
            if (this.currentPageNumber > change.currentValue) {
                // throws ExpressionChangedAfterItHasBeenCheckedException
                // if there's no setTimeout.
                // no need to add setTimeout if ngOnChanges 
                // is fired after changes made on root component.
                setTimeout(function () { return _this.setCurrentPage(1); }, 1);
            }
        }
    };
    AcDataGridPagination.prototype.setCurrentPage = function (pageNumber, event) {
        if (event) {
            event.preventDefault();
        }
        if (pageNumber === 0 || pageNumber > this.maxPageIndex
            || pageNumber === this.currentPageNumber) {
            return;
        }
        this.pageNumberChanged.emit(pageNumber);
        this.currentPageNumber = pageNumber;
    };
    AcDataGridPagination.prototype.range = function (min, max) {
        var result = new Array();
        for (var i = min; i <= max; i++) {
            result.push(i);
        }
        return result;
    };
    Object.defineProperty(AcDataGridPagination.prototype, "pageStartNumber", {
        get: function () {
            var startNumber = this.currentPageNumber <= 4
                ? 1
                : this.currentPageNumber >= this.maxPageIndex - 3
                    ? this.maxPageIndex - 6
                    : this.currentPageNumber - 3;
            return startNumber < 1 ? 1 : startNumber;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AcDataGridPagination.prototype, "pageEndNumber", {
        get: function () {
            var pageEnd = this.pageStartNumber + 6;
            return pageEnd > this.maxPageIndex ? this.maxPageIndex : pageEnd;
        },
        enumerable: true,
        configurable: true
    });
    return AcDataGridPagination;
}());
__decorate([
    Input(),
    __metadata("design:type", Number)
], AcDataGridPagination.prototype, "maxPageIndex", void 0);
__decorate([
    Output(),
    __metadata("design:type", Object)
], AcDataGridPagination.prototype, "pageNumberChanged", void 0);
AcDataGridPagination = __decorate([
    Component({
        selector: 'ac-pagination',
        styles: [
            ".pagination {\n      display: inline-block;\n      padding-left: 0;\n      margin: 20px 0;\n      border-radius: 4px;\n    }",
            ".pagination > li {\n      display: inline;\n    }",
            ".pagination > li:first-child > span {\n      margin-left: 0;\n      border-bottom-left -radius: 4px;\n      border-top-left-radius: 4px;\n    }",
            ".pagination >.disabled > a, .pagination >.disabled > a:focus, .pagination >.disabled > a:hover, .pagination >.disabled > span, .pagination >.disabled > span:focus, .pagination >.disabled > span:hover {\n      color: #777;\n      background-color: #fff;\n      border-color: #ddd;\n      cursor: not-allowed;\n    }",
            ".pagination > li > a, .pagination > li > span {\n      position: relative;\n      float: left;\n      padding: 6px 12px;\n      line-height: 1.42857;\n      text-decoration: none;\n      color: #337ab7;\n      background-color: #fff;\n      border: 1px solid #ddd;\n      margin-left: -1px;\n    }"
        ],
        template: "\n    <div>\n        <ul class=\"pagination float-right\">\n            <li [class.disabled]=\"currentPageNumber === 1 || !maxPageIndex\">\n                <a href (click)=\"setCurrentPage(1, $event)\" aria-label=\"Previous\">\n                    <span aria-hidden=\"true\">\u00AB</span>\n                </a>\n            </li>\n            <li [class.disabled]=\"currentPageNumber === 1 || !maxPageIndex\">\n                <a href aria-label=\"Previous\" \n                    (click)=\"setCurrentPage(currentPageNumber - 1, $event)\">\n                    <span aria-hidden=\"true\">\u2039</span>\n                </a>\n            </li>\n            <li *ngFor=\"let index of range(pageStartNumber, pageEndNumber)\" \n                        [class.active]=\"currentPageNumber === index\">\n                <a href (click)=\"setCurrentPage(index, $event)\">\n                    <span aria-hidden=\"true\">{{ index }}</span>\n                </a>\n            </li>\n            <li [class.disabled]=\"currentPageNumber === maxPageIndex \n                                  || !maxPageIndex\">\n                <a href (click)=\"setCurrentPage(currentPageNumber + 1, $event)\" aria-label=\"Last\">\n                    <span aria-hidden=\"true\">\u203A</span>\n                </a>\n            </li>\n            <li [class.disabled]=\"currentPageNumber === maxPageIndex \n                                  || !maxPageIndex\">\n                <a href (click)=\"setCurrentPage(maxPageIndex, $event)\" aria-label=\"Last\">\n                    <span aria-hidden=\"true\">\u00BB</span>\n                </a>\n            </li>\n        </ul>\n    </div>    \n    "
    })
], AcDataGridPagination);
export { AcDataGridPagination };

//# sourceMappingURL=ac-datagridpagination.component.js.map
