import { OnInit } from '@angular/core';
import { FormGroup } from '@angular/forms';
/**
 * The Editor Widget. Creates a field with all required validators using decorators and forms service.
 */
export declare class AcEditor implements OnInit {
    /**
     * Field name
     */
    name: string;
    /**
     * Editor type. Default is 'text';
     */
    type: string;
    /**
     * A character after the fields label. Default is ': ' (colon plus space);
     */
    labelDivider: string;
    /**
     * The label's name.
     */
    label: string;
    /**
     * A tooltip
     */
    tooltip: string;
    /**
     * Name of the form's group object.
     */
    userForm: FormGroup;
    /**
     * If set to true the label and the field appears in one row.
     * Otherwise the label is above the field. Default is `true`.
     */
    inline: boolean;
    /**
     * The values of the select field provided by an enum. For other fieldtypes it's being ignored.
     */
    enumValues: any;
    /**
     * The values of the select field provided by a list. For other fieldtypes it's being ignored.
     * The value shall be an Array that a `*ngFor` directive can execute.
     */
    listValues: Array<any>;
    /**
     * The start value for a range field. Other field types ignore this value.
     */
    fromValue: number;
    /**
     * The end value for a range field. Other field types ignore this value.
     */
    toValue: number;
    /**
     * An optional placeholder for empty field. The default is empty (no watermark).
     */
    waterMark: string;
    /**
     * Renders the field as read only.
     */
    readonly: boolean;
    /**
     * The value set to and read from the field.
     */
    value: any;
    params: {
        key: string;
        value: any;
    }[];
    errors: Array<string>;
    constructor();
    getParams(key: string): any;
    first(array: Array<{
        key: string;
        value: any;
    }>, n: number): any;
    ngOnInit(): void;
    private onValueChanged(data);
}
