var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Component, Input, Output } from '@angular/core';
import { FormGroup } from '@angular/forms';
/**
 * The Editor Widget. Creates a field with all required validators using decorators and forms service.
 */
var AcEditor = (function () {
    function AcEditor() {
        /**
         * Editor type. Default is 'text';
         */
        this.type = 'text';
        /**
         * A character after the fields label. Default is ': ' (colon plus space);
         */
        this.labelDivider = ': ';
        /**
         * If set to true the label and the field appears in one row.
         * Otherwise the label is above the field. Default is `true`.
         */
        this.inline = true;
        /**
         * The start value for a range field. Other field types ignore this value.
         */
        this.fromValue = 0;
        /**
         * The end value for a range field. Other field types ignore this value.
         */
        this.toValue = 100;
        /**
         * An optional placeholder for empty field. The default is empty (no watermark).
         */
        this.waterMark = '';
        /**
         * Renders the field as read only.
         */
        this.readonly = false;
        this.params = new Array();
    }
    AcEditor.prototype.getParams = function (key) {
        return this.first(this.params.filter(function (e) { return e.key === key; }), 0);
    };
    AcEditor.prototype.first = function (array, n) {
        if (array == null) {
            return void 0;
        }
        if (n == null) {
            return array[0];
        }
        if (n < 0) {
            return [];
        }
        return array.slice(0, n);
    };
    AcEditor.prototype.ngOnInit = function () {
        var _this = this;
        this.userForm.valueChanges.subscribe(function (data) { return _this.onValueChanged(data); });
        // this is set by FormValidatorService
        var editorModel = this.userForm['__editorModel__'];
        // get type from form
        if (editorModel) {
            // get elementary types, this might get overwritten later according to decorators found
            if (typeof editorModel[this.name] === 'string') {
                this.type = 'text';
            }
            if (typeof editorModel[this.name] === 'boolean') {
                this.type = 'boolean';
            }
            if (typeof editorModel[this.name] === 'number') {
                this.type = 'number';
            }
            if (editorModel[this.name] instanceof Date) {
                this.type = 'calendar';
            }
            // make an instance to read the properties
            this.label = editorModel["__displayName__" + this.name] || this.label || this.name;
            this.tooltip = editorModel["__displayDesc__" + this.name] || this.tooltip || this.name;
            // render as range id there is a range definition
            if (editorModel["__hasRangeFrom__" + this.name] && Number(editorModel["__hasRangeFrom__" + this.name])) {
                this.fromValue = editorModel["__hasRangeFrom__" + this.name];
                this.type = 'range';
            }
            if (editorModel["__hasRangeTo__" + this.name] && Number(editorModel["__hasRangeTo__" + this.name])) {
                this.toValue = editorModel["__hasRangeTo__" + this.name];
                this.type = 'range';
            }
            // placeholder
            if (editorModel["__hasWatermark__" + this.name]) {
                this.waterMark = editorModel["__watermark__" + this.name];
            }
            // templates
            if (editorModel["__hasTemplateHint__" + this.name]) {
                this.type = editorModel["__templateHint__" + this.name].toLowerCase();
                if (editorModel["__templateHintParams__" + this.name]) {
                    this.params = editorModel["__templateHintParams__" + this.name];
                }
            }
            // render hidden fields as hidden even in forms
            if (editorModel["__isHidden__" + this.name]) {
                this.type = 'hidden';
            }
            // check readonly
            if (editorModel["__isReadonly__" + this.name]) {
                this.readonly = !!editorModel["__isReadonly__" + this.name];
            }
        }
    };
    AcEditor.prototype.onValueChanged = function (data) {
        // check validation on change
        this.errors = new Array();
        for (var error in this.userForm.controls[this.name].errors) {
            this.errors.push(error);
        }
    };
    return AcEditor;
}());
__decorate([
    Input(),
    __metadata("design:type", String)
], AcEditor.prototype, "name", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], AcEditor.prototype, "type", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], AcEditor.prototype, "labelDivider", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], AcEditor.prototype, "label", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], AcEditor.prototype, "tooltip", void 0);
__decorate([
    Input(),
    __metadata("design:type", FormGroup)
], AcEditor.prototype, "userForm", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], AcEditor.prototype, "inline", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], AcEditor.prototype, "enumValues", void 0);
__decorate([
    Input(),
    __metadata("design:type", Array)
], AcEditor.prototype, "listValues", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], AcEditor.prototype, "fromValue", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], AcEditor.prototype, "toValue", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], AcEditor.prototype, "waterMark", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], AcEditor.prototype, "readonly", void 0);
__decorate([
    Output(),
    Input(),
    __metadata("design:type", Object)
], AcEditor.prototype, "value", void 0);
AcEditor = __decorate([
    Component({
        selector: 'ac-editor',
        styles: [
            'input[type="checkbox"] { display: none; }',
            'input[type="checkbox"] + label:before { font-family: FontAwesome; }',
            'input[type="checkbox"] + label:before { content: "\\f096"; }',
            'input[type="checkbox"]:checked + label:before { content: "\\f046"; }',
            "input[type=\"checkbox\"] + label { \n      display:inline-block; width:15px; height: 20px; margin: -1px 4px 0 0; vertical-align:middle; cursor: pointer; \n    }",
        ],
        template: "<div class=\"form-group row\" [formGroup]=\"userForm\" *ngIf=\"type != 'hidden'\" [ngClass]=\"{ 'has-danger': !userForm.controls[name].valid && userForm.controls[name].touched }\"><label class=\"col-form-label col-md-3 col-sm-10\" [attr.for]=\"name\" [attr.title]=\"tooltip\">{{ label }}{{ labelDivider }}</label><div [ngClass]=\"{ 'col-md-7 col-sm-10': inline }\"><textarea *ngIf=\"type == 'textarea'\" class=\"form-control\" [id]=\"name\" [readOnly]=\"readonly\" [formControlName]=\"name\" [attr.rows]=\"getParams('rows')\" [attr.cols]=\"getParams('cols')\"> </textarea><select *ngIf=\"type == 'enum' && enumValues\" class=\"form-control\" [id]=\"name\" [formControlName]=\"name\" [disabled]=\"readonly\"><option *ngFor=\"let option of enumValues\" [value]=\"option.key\">{{option.val}}</option></select><select *ngIf=\"type == 'list' && listValues\" class=\"form-control\" [id]=\"name\" [formControlName]=\"name\" [disabled]=\"readonly\"><option *ngFor=\"let option of listValues\" [value]=\"option.key\">{{option.val}}</option></select><div class=\"form-check\" *ngIf=\"type == 'radio' && (listValues || enumValues)\"><label class=\"form-check-label\" *ngIf=\"listValues\"><input type=\"radio\" *ngFor=\"let option of listValues\" [id]=\"name\" [formControlName]=\"name\" [disabled]=\"readonly\" [value]=\"option.key\">{{option.val}}</label><label class=\"form-check-label\" *ngIf=\"enumValues\"><input type=\"radio\" *ngFor=\"let option of enumValues\" [id]=\"name\" [formControlName]=\"name\" [disabled]=\"readonly\" [value]=\"option.key\">{{option.val}}</label></div><input *ngIf=\"type == 'range'\" type=\"range\" class=\"form-control\" [placeholder]=\"waterMark\" [attr.minvalue]=\"fromValue\" [attr.maxvalue]=\"toValue\" [attr.min]=\"fromValue\" [attr.max]=\"toValue\" [id]=\"name\" [formControlName]=\"name\" [title]=\"tooltip\"> <input *ngIf=\"type == 'calendar'\" type=\"date\" class=\"form-control\" [placeholder]=\"waterMark\" [readOnly]=\"readonly\" [id]=\"name\" [formControlName]=\"name\" [title]=\"tooltip\"> <input *ngIf=\"type == 'number'\" type=\"number\" class=\"form-control\" [placeholder]=\"waterMark\" [readOnly]=\"readonly\" [id]=\"name\" [formControlName]=\"name\" [title]=\"tooltip\"> <input *ngIf=\"type == 'boolean'\" type=\"checkbox\" class=\"form-control\" [readOnly]=\"readonly\" [id]=\"name\" [formControlName]=\"name\" [title]=\"tooltip\"><label *ngIf=\"type == 'boolean'\" [attr.for]=\"name\"></label><input *ngIf=\"type == 'text' || type == ''\" [placeholder]=\"waterMark\" [readOnly]=\"readonly\" type=\"text\" class=\"form-control\" [id]=\"name\" [formControlName]=\"name\"><ng-container *ngIf=\"type == 'template'\"><ng-content></ng-content></ng-container><span class=\"fa fa-warning text-danger form-control-feedback\" [hidden]=\"userForm.controls[name].valid || userForm.controls[name].pristine\"></span> <small class=\"text-danger\" [hidden]=\"userForm.controls[name].valid || userForm.controls[name].pristine\"><span *ngFor=\"let error of errors\">{{ userForm.controls[name].messages[error] }}</span></small></div></div><div [formGroup]=\"userForm\"><input *ngIf=\"type == 'hidden'\" [id]=\"name\" [formControlName]=\"name\" type=\"hidden\"></div>"
    }) //
    ,
    __metadata("design:paramtypes", [])
], AcEditor);
export { AcEditor };

//# sourceMappingURL=ac-editor.component.js.map
