var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Component, Input } from '@angular/core';
import { InputConverter, EnumConverter } from '../../../core/utils/convert-inputconverter';
import { AcInfoBoxOptions } from './models/options-infobox';
import { Meaning } from '../../../core/utils/enum-colors';
/**
 * The InfoBox is a rectangle area to show dynamic data. It has icon, text, and additional information.
 *
 * @param icon The name of an FontAwesome icon, such as "fa-circle"
 * @param text The text that appears prominently
 * @param number A number that covers the element as the value
 * @param footerText A smaller explanation text at the end
 * @param footerLink A link, suitable for [routerLink] that is being invoked from the footerText
 * @param progressValue A progress Value that creates a progress bar position
 * @param progressText A progress Text that replaces the footerText
 * @param color The background color
 */
var AcInfoBox = (function () {
    function AcInfoBox() {
        this.color = Meaning.Info;
        this.text = 'Demo';
        this.progressValue = 0;
        this.progressText = '';
        this.footerLink = '/';
        this.footerText = '';
        this.icon = 'fa-user';
    }
    AcInfoBox.prototype.ngOnInit = function () {
        if (!this.options) {
            this.options = new AcInfoBoxOptions();
            this.options.hasFooter = !!this.footerLink || !!this.footerText;
            this.options.hasProgress = !!this.progressValue || !!this.progressText;
        }
    };
    AcInfoBox.prototype.getColor = function (type) {
        if (this.color) {
            var color = EnumConverter(this.color, Meaning).toLowerCase();
            return type + "-" + color;
        }
    };
    return AcInfoBox;
}());
__decorate([
    Input(),
    __metadata("design:type", String)
], AcInfoBox.prototype, "icon", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], AcInfoBox.prototype, "text", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], AcInfoBox.prototype, "number", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], AcInfoBox.prototype, "footerText", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], AcInfoBox.prototype, "footerLink", void 0);
__decorate([
    Input(),
    __metadata("design:type", Number)
], AcInfoBox.prototype, "progressValue", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], AcInfoBox.prototype, "progressText", void 0);
__decorate([
    Input(),
    InputConverter(EnumConverter, Meaning) // need EnumConverter, as otherwise metadata returns Number
    ,
    __metadata("design:type", Number)
], AcInfoBox.prototype, "color", void 0);
__decorate([
    Input(),
    __metadata("design:type", AcInfoBoxOptions)
], AcInfoBox.prototype, "options", void 0);
AcInfoBox = __decorate([
    Component({
        selector: 'ac-infobox',
        template: "<div class=\"card card-inverse\" [ngClass]=\"getColor('card')\">\n                  <div class=\"card-body\" [ngClass]=\"getColor('bg')\">\n                    <div class=\"rotate\">\n                        <i class=\"fa fa-5x\" [ngClass]=\"icon\"></i>\n                    </div>\n                    <h6 class=\"text-uppercase\">{{ text }}</h6>\n                    <h1 class=\"display-1\">{{ number }}</h1>\n                    <div class=\"progress\" *ngIf=\"options.hasProgress\">\n                        <div class=\"progress-bar\" [style.width]=\"progressValue\"></div>\n                    </div>\n                    <span class=\"progress-description\" *ngIf=\"options.hasProgress\">\n                        {{progressText}}\n                    </span>\n                  </div>\n                  <div class=\"card-footer text-muted\" *ngIf=\"options.hasFooter\">\n                      <a href=\"#\" class=\"card-link\" [routerLink]=\"footerLink\">{{footerText}}</a>\n                  </div>\n                </div>"
    }) //
    ,
    __metadata("design:paramtypes", [])
], AcInfoBox);
export { AcInfoBox };

//# sourceMappingURL=ac-infobox.component.js.map
