var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Component, Input, Output, EventEmitter, ElementRef, Renderer } from '@angular/core';
import { AcTextTreeNode, AcTreeNodeState } from './models/index';
/**
 * The treenode class represents a single treenode for the treeview. Actually, this renders the real UI.
 *
 * The component needs this input value:
 *
 * * @Input() node: AcTextTreeNode;
 *
 * The several options are set through the AcTreeNodeOptions object that is being held by the
 * AcTextTreeNode class. This includes colors, icons, and other styles.
 *
 * The component can fire these events:
 *
 * * @Output() nodeClick: EventEmitter<AcTreeNode>;
 * * @Output() checkChanged: EventEmitter<AcTreeNode>;
 * * @Output() selectedChanged: EventEmitter<AcTreeNode>;
 * * @Output() collapseChanged: EventEmitter<AcTreeNode>;
 *
 */
var AcTreeViewNode = AcTreeViewNode_1 = (function () {
    function AcTreeViewNode(el, renderer) {
        this.el = el;
        this.renderer = renderer;
        this.isExpanded = false;
        this.nodeClick = new EventEmitter();
        this.checkChanged = new EventEmitter();
        this.selectedChanged = new EventEmitter();
        this.collapseChanged = new EventEmitter();
    }
    AcTreeViewNode.prototype.ngOnInit = function () {
        var _this = this;
        // set HTML according to options
        this.collapseClasses = new Array();
        this.iconClasses = new Array();
        // expander area with icon
        if (this.node) {
            // expect a font-awesome class with or without fa class
            if (this.node.options && this.node.options.icon) {
                if (this.node.options.icon.indexOf(AcTreeViewNode_1.pfxIcon + " ") !== 0) {
                    this.iconClasses.push('fa');
                }
                this.iconClasses.push(this.node.options.icon);
            }
            if (this.node.options && this.node.options.href) {
                this.href = this.node.options.href;
            }
            else {
                this.href = '';
            }
            // open/close area for elements with children
            if (this.node.hasChildren) {
                this.collapseClasses.push('ac-icon'); // base class
                this.collapseClasses.push(AcTreeViewNode_1.pfxIcon);
                this.collapseClasses.push(AcTreeViewNode_1.opnIcon);
                // collapsed by default
                this.node.state &= ~AcTreeNodeState.checked;
            }
            if (this.node.options && this.node.options.color) {
                this.foreColor = this.node.options.color;
            }
            if (this.node.options && this.node.options.backColor) {
                this.backColor = this.node.options.backColor;
            }
            this.node.stateChange.subscribe(function (se) {
                _this.collapseChanged.emit(_this.node);
                if (!_this.node.stateIsExpandend) {
                    _this.collapseClasses = _this.collapseClasses.filter(function (s) { return s != AcTreeViewNode_1.clsIcon; });
                    _this.collapseClasses.push(AcTreeViewNode_1.opnIcon);
                }
                else {
                    _this.collapseClasses = _this.collapseClasses.filter(function (s) { return s != AcTreeViewNode_1.opnIcon; });
                    _this.collapseClasses.push(AcTreeViewNode_1.clsIcon);
                }
                _this.isExpanded = _this.node.stateIsExpandend;
            });
        }
    };
    // forward events in the node tree
    AcTreeViewNode.prototype.onNodeClick = function (node) {
        this.nodeClick.emit(node);
    };
    AcTreeViewNode.prototype.onCheckChanged = function (node) {
        this.checkChanged.emit(node);
    };
    AcTreeViewNode.prototype.onSelectedChanged = function (node) {
        this.selectedChanged.emit(node);
    };
    AcTreeViewNode.prototype.onCollapseChanged = function (node) {
        if (this.node.options.collapsable && this.node.hasChildren) {
            this.collapseChanged.emit(node);
        }
    };
    AcTreeViewNode.prototype.handleCheckChange = function () {
        if (this.node.options.checkable && !this.node.stateIsDisabled) {
            if (this.node.state & AcTreeNodeState.checked) {
                this.node.state &= ~AcTreeNodeState.checked;
            }
            else {
                this.node.state |= AcTreeNodeState.checked;
            }
            this.checkChanged.emit(this.node);
        }
    };
    AcTreeViewNode.prototype.handleClick = function ($event) {
        $event.stopPropagation();
        // always emit click
        this.nodeClick.emit(this.node);
    };
    AcTreeViewNode.prototype.handleCollapse = function () {
        // if collapsable handle icons and view state
        if (this.node.options.collapsable) {
            // toggle state
            if (this.node.state && (this.node.state & AcTreeNodeState.expanded)) {
                this.node.state &= ~AcTreeNodeState.expanded;
            }
            else {
                this.node.state |= AcTreeNodeState.expanded;
            }
        }
    };
    AcTreeViewNode.prototype.handlePreSelection = function (state) {
        // handle the states according to options
        if (this.node.options.selectable && !this.node.stateIsDisabled) {
            if (state) {
                if (this.node.options && this.node.options.color) {
                    this.foreColor = this.node.options.backColor;
                }
                else {
                    this.foreColor = '#fff';
                }
                if (this.node.options && this.node.options.backColor) {
                    this.backColor = this.node.options.color;
                }
                else {
                    this.backColor = '#000';
                }
            }
            else {
                if (this.node.options && this.node.options.color) {
                    this.foreColor = this.node.options.color;
                }
                else {
                    this.foreColor = '#000';
                }
                if (this.node.options && this.node.options.backColor) {
                    this.backColor = this.node.options.backColor;
                }
                else {
                    this.backColor = '#fff';
                }
            }
            this.preSelectState = state;
        }
    };
    AcTreeViewNode.prototype.handleSelection = function ($event) {
        // handle the states according to options
        if (this.node.options.selectable && !this.node.stateIsDisabled && this.preSelectState === true) {
            if (this.node.state & AcTreeNodeState.selected) {
                this.node.state &= ~AcTreeNodeState.selected;
            }
            else {
                this.node.state |= AcTreeNodeState.selected;
            }
            if (this.node.stateIsSelected) {
                if (this.node.options && this.node.options.color) {
                    this.foreColor = this.node.options.backColor;
                }
                else {
                    this.foreColor = '#fff';
                }
                if (this.node.options && this.node.options.backColor) {
                    this.backColor = this.node.options.color;
                }
                else {
                    this.backColor = '#000';
                }
            }
            else {
                if (this.node.options && this.node.options.color) {
                    this.foreColor = this.node.options.color;
                }
                else {
                    this.foreColor = '#000';
                }
                if (this.node.options && this.node.options.backColor) {
                    this.backColor = this.node.options.backColor;
                }
                else {
                    this.backColor = '#fff';
                }
            }
            this.selectedChanged.emit(this.node);
        }
        if (!this.href) {
            $event.preventDefault();
            $event.stopPropagation();
        }
    };
    return AcTreeViewNode;
}());
AcTreeViewNode.pfxIcon = 'fa';
AcTreeViewNode.opnIcon = 'fa-plus';
AcTreeViewNode.clsIcon = 'fa-minus';
__decorate([
    Input(),
    __metadata("design:type", AcTextTreeNode)
], AcTreeViewNode.prototype, "node", void 0);
__decorate([
    Output(),
    __metadata("design:type", EventEmitter)
], AcTreeViewNode.prototype, "nodeClick", void 0);
__decorate([
    Output(),
    __metadata("design:type", EventEmitter)
], AcTreeViewNode.prototype, "checkChanged", void 0);
__decorate([
    Output(),
    __metadata("design:type", EventEmitter)
], AcTreeViewNode.prototype, "selectedChanged", void 0);
__decorate([
    Output(),
    __metadata("design:type", EventEmitter)
], AcTreeViewNode.prototype, "collapseChanged", void 0);
AcTreeViewNode = AcTreeViewNode_1 = __decorate([
    Component({
        selector: 'ac-treenode',
        template: "<li class=\"treeview\" (click)=\"handleClick($event)\"><i class=\"ac-collapse\" [ngClass]=\"collapseClasses\" *ngIf=\"node.hasChildren\" (click)=\"handleCollapse()\"></i> <i class=\"ac-collapse\" *ngIf=\"!node.hasChildren\"></i> <i class=\"ac-icon\" [ngClass]=\"iconClasses\" [style.color]=\"node.options.iconColor\" *ngIf=\"!node.options.checkable\"></i> <input type=\"checkbox\" [id]=\"node.name\" *ngIf=\"node.options.checkable\" [checked]=\"node.stateIsChecked\" (click)=\"handleCheckChange()\"><label [attr.for]=\"node.name\"></label><a class=\"ac-container\" [href]=\"href\" [style.color]=\"foreColor\" [style.background-color]=\"backColor\" (mouseover)=\"handlePreSelection(true)\" (mouseout)=\"handlePreSelection(false)\" (click)=\"handleSelection($event)\">{{ node.text }}</a><ul class=\"treeview\" *ngIf=\"node.hasChildren\" [hidden]=\"!isExpanded\"><ac-treenode *ngFor=\"let child of node.children\" [node]=\"child\" (nodeClick)=\"onNodeClick($event)\" (checkChanged)=\"onCheckChanged($event)\" (selectedChanged)=\"onSelectedChanged($event)\" (collapseChanged)=\"onCollapseChanged($event)\"></ac-treenode></ul></li>",
        styles: ["ul.treeview{list-style:none;margin-left:-2em}li.treeview{margin-left:10px;margin-bottom:3px;box-sizing:border-box}li.treeview a.ac-container{border-radius:2px;display:inline-block;padding:3px;text-decoration:none}li.treeview input[type=checkbox]{display:none}li.treeview input[type=checkbox]+label:before{font-family:FontAwesome}li.treeview input[type=checkbox]+label:before{content:\"\\f096\"}li.treeview input[type=checkbox]:checked+label:before{content:\"\\f046\"}li.treeview input[type=checkbox]+label{display:inline-block;width:15px;height:20px;margin:-1px 4px 0 0;vertical-align:middle;cursor:pointer}li.treeview i.ac-collapse{width:15px;cursor:pointer;display:inline-block;margin-left:-1.7em}li.treeview i.ac-icon{width:15px;cursor:pointer;display:inline-block}li.treeview .ac-node-disabled{color:silver;cursor:not-allowed}"]
    }),
    __metadata("design:paramtypes", [ElementRef, Renderer])
], AcTreeViewNode);
export { AcTreeViewNode };
var AcTreeViewNode_1;

//# sourceMappingURL=ac-treeview-node.component.js.map
