import { EventEmitter } from '@angular/core';
import * as tree from './ac-treenodeoptions';
/**
 * The base class.
 */
var AcTreeNode = (function () {
    function AcTreeNode(options, nodes) {
        this.options = options || new tree.AcTreeNodeOptions();
        if (nodes && nodes instanceof Array) {
            this.children = nodes;
        }
        else {
            if (nodes) {
                this.children = [nodes];
            }
        }
        this.stateChange = new EventEmitter();
        this.state = tree.AcTreeNodeState.undefined;
    }
    Object.defineProperty(AcTreeNode.prototype, "path", {
        get: function () {
            // walk up tree and return path with names
            var p = new Array();
            p.push(this);
            var n = this;
            while (this.parent) {
                n = n.parent;
                p.push(n);
            }
            return p;
        },
        enumerable: true,
        configurable: true
    });
    AcTreeNode.prototype.hasDirectAncestor = function (node) {
        return this.parent !== undefined;
    };
    Object.defineProperty(AcTreeNode.prototype, "hasChildren", {
        get: function () {
            return this.children !== undefined && this.children.length > 0;
        },
        enumerable: true,
        configurable: true
    });
    AcTreeNode.prototype.add = function (nodes) {
        if (nodes && nodes instanceof Array) {
            (_a = this.children).push.apply(_a, nodes);
        }
        else {
            this.children.push(nodes);
        }
        var _a;
    };
    AcTreeNode.prototype.remove = function (node) {
        var idx = this.children.indexOf(node);
        if (idx > -1) {
            this.children.splice(idx, 1);
            return true;
        }
        else {
            return false;
        }
    };
    Object.defineProperty(AcTreeNode.prototype, "state", {
        get: function () {
            return this._state;
        },
        set: function (value) {
            this._state = value;
            this.stateChange.emit(this._state);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AcTreeNode.prototype, "stateIsExpandend", {
        get: function () {
            return (this.state & tree.AcTreeNodeState.expanded) === tree.AcTreeNodeState.expanded;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AcTreeNode.prototype, "stateIsDisabled", {
        get: function () {
            return (this.state & tree.AcTreeNodeState.disabled) === tree.AcTreeNodeState.disabled;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AcTreeNode.prototype, "stateIsSelected", {
        get: function () {
            return (this.state & tree.AcTreeNodeState.selected) === tree.AcTreeNodeState.selected;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AcTreeNode.prototype, "stateIsChecked", {
        get: function () {
            return (this.state & tree.AcTreeNodeState.checked) === tree.AcTreeNodeState.checked;
        },
        enumerable: true,
        configurable: true
    });
    return AcTreeNode;
}());
export { AcTreeNode };

//# sourceMappingURL=ac-treenode.js.map
